# -*- coding: utf-8 -*-

from rest_framework import serializers
from clariweb.clariweb.models import (
    Notification,
    AccessLog,
    DocumentStatisticsPerMonth,
    PaymentResponse,
    ClarilabPaymentResponse,
)


class NotificationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Notification
        fields = ('date', 'status', 'sms', 'email', 'origin')


class AccessLogSerializer(serializers.ModelSerializer):
    class Meta:
        model = AccessLog
        fields = ('date', 'user', 'message', 'ip', 'full_name')
    full_name = serializers.SerializerMethodField('get_user_full_name')
    message = serializers.SerializerMethodField('get_message_display')

    def get_message_display(self, obj):
        return obj.get_message_display()

    def get_user_full_name(self, obj):
        if obj.user:
            return '(%s) %s' % (obj.username, obj.user.get_full_name())
        elif obj.username:
            return obj.username
        else:
            return 'NA'


class SMSBuySerializer(serializers.Serializer):
    quantity = serializers.IntegerField()

    def validate_quantity(self, value):
        if value not in (100, 1000, 10000):
            raise serializers.ValidationError("Invalid quantity.")
        return value


class UsernameSerializer(serializers.Serializer):
    username = serializers.CharField(max_length=128)


class UserModifySerializer(serializers.Serializer):
    username = serializers.CharField(max_length=128)
    password = serializers.CharField(max_length=128, required=False)
    is_active = serializers.BooleanField(required=False)

    def validate_password(self, value):
        if value is not None and len(value) < 8:
            raise serializers.ValidationError("Password too short.")
        return value


class DocumentStatisticsPerMonthPerUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = DocumentStatisticsPerMonth
        fields = ('timestep', 'user', 'documents', 'username')
    user = serializers.SerializerMethodField('get_user_full_name')
    username = serializers.SerializerMethodField('get_shortusername')

    def get_shortusername(self, obj):
        if obj.user and obj.user.user:
            return obj.user.user.username
        else:
            return None

    def get_user_full_name(self, obj):
        if obj.user and obj.user.user:
            return obj.user.user.get_full_name()
        else:
            return ''


class DocumentStatisticsPerMonthGraphSerializer(serializers.ModelSerializer):
    class Meta:
        model = DocumentStatisticsPerMonth
        fields = ('timestep', 'documents', 'userkind')

    userkind = serializers.CharField(source='user__kind', read_only=True)


class PaymentResponseSerializer(serializers.ModelSerializer):
    class Meta:
        model = PaymentResponse
        fields = (
            'vads_amount',
            'vads_trans_id',
            'date',
            'status',
            'clarilab_success',
            'clarilab_reason',
            'name',
        )
    date = serializers.SerializerMethodField()
    status = serializers.SerializerMethodField()
    clarilab_success = serializers.SerializerMethodField()
    clarilab_reason = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()

    def get_date(self, pr):
        if pr.request:
            return pr.request.date_insert
        return None

    def get_clarilab_success(self, pr):
        try:
            return pr.clarilabpaymentresponse.success
        except ClarilabPaymentResponse.DoesNotExist:
            return None

    def get_clarilab_reason(self, pr):
        try:
            return pr.clarilabpaymentresponse.reason
        except ClarilabPaymentResponse.DoesNotExist:
            return None

    def get_name(self, pr):
        name = []
        request = pr.request
        if request:
            if request.vads_cust_title:
                name.append(request.vads_cust_title)
            if request.vads_cust_last_name:
                name.append(request.vads_cust_last_name)
            if request.vads_cust_first_name:
                name.append(request.vads_cust_first_name)
        return u' '.join(name)

    def get_status(self, pr):
        if pr.payment_successful:
            return "paid"
        else:
            return "refused"
