# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import six
import datetime
import ipaddress
from constance import settings as constance_settings
from constance import config as constance_config
from django.db.models import Q, Sum
from django.core.mail import send_mail
from django.contrib.auth import (
    logout, HASH_SESSION_KEY, SESSION_KEY)
from django.core.signing import TimestampSigner, SignatureExpired
from rest_framework import filters, generics, viewsets, status, mixins
from rest_framework.response import Response
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.views import APIView
from clariweb.backend.tasks import update_sms_quota
from clariweb.clariweb.smscount import SMSCounter
from clariweb.clariweb.models import (
    Notification,
    BasicUser,
    ClariwebUser,
    AccessLog,
    DocumentStatisticsPerMonth,
    PaymentResponse,
)
from clariweb.api.serializers import FullClariwebUserSerializer
from clariweb.api.admin.serializers import (
    NotificationSerializer,
    SMSBuySerializer,
    UsernameSerializer,
    UserModifySerializer,
    AccessLogSerializer,
    DocumentStatisticsPerMonthGraphSerializer,
    DocumentStatisticsPerMonthPerUserSerializer,
    PaymentResponseSerializer,
)
from clariweb.api.filters import SearchFilter
from clariweb.api.pagination import (
    StandardPagination, NoCountPagination, LoginAsUserPagination)


class NotificationFilteringView(generics.ListAPIView):
    queryset = Notification.objects.all().select_related("user__user")
    serializer_class = NotificationSerializer
    permission_classes = (IsAdminUser, )
    filter_backends = (SearchFilter, filters.OrderingFilter)
    search_fields = ('=status', '^user__user__username', '^sms', '^email')
    ordering = ('-date', 'status', 'user__user__username', 'sms', 'email')
    pagination_class = NoCountPagination


class AccessLogFilteringView(generics.ListAPIView):
    queryset = AccessLog.objects.all().select_related("user__clariwebuser")
    serializer_class = AccessLogSerializer
    permission_classes = (IsAdminUser, )
    filter_backends = (SearchFilter,)
    search_fields = ('date', '^message', '^username^', '^ip',
                     '^user__first_name', '^user__last_name')
    ordering = ('-date',)
    pagination_class = NoCountPagination


class PaymentResponseFilteringView(generics.ListAPIView):
    queryset = PaymentResponse.objects.all()
    serializer_class = PaymentResponseSerializer
    permission_classes = (IsAdminUser, )
    filter_backends = (SearchFilter, filters.OrderingFilter)
    search_fields = ('^vads_trans_id', '^pzrequest__vads_cust_last_name^',
                     '^pzrequest__vads_cust_first_name^')
    ordering = ('-pzrequest__date_insert', 'vads_trans_id')
    ordering_fields = ('pzrequest__date_insert', 'vads_result',
                       'clarilabpaymentresponse__status')
    pagination_class = NoCountPagination


class DocumentStatisticsPerMonthPerUserView(generics.ListAPIView):
    queryset = DocumentStatisticsPerMonth.objects.all() \
        .prefetch_related("user__user")
    serializer_class = DocumentStatisticsPerMonthPerUserSerializer
    permission_classes = (IsAdminUser, )
    ordering = ('-timestep', '-documents')
    pagination_class = StandardPagination


class DocumentStatisticsPerMonthGraphView(generics.ListAPIView):
    queryset = DocumentStatisticsPerMonth.objects.all() \
        .values("timestep", "user__kind") \
        .annotate(documents=Sum('documents')) \
        .order_by("timestep", "user__kind")
    serializer_class = DocumentStatisticsPerMonthGraphSerializer
    permission_classes = (IsAdminUser, )


class SMSCountView(viewsets.ViewSet):
    permission_classes = (IsAdminUser, )

    def get_sms_sent(self, request):
        q = (Q(status="SENT") | Q(status="CONFIRMED"))
        q &= ~Q(sms='') & ~Q(sms=None)
        smscount_total = Notification.objects.filter(q).count()
        return Response({"smscount_total": smscount_total})

    def get_sms_quota(self, request):
        return Response({"sms_quota": SMSCounter.get_quota_left()})

    def update_sms_quota(self, request):
        try:
            update_sms_quota()
        except Exception as exc:
            return Response(
                {"errors": "{}".format(exc)}, status=status.HTTP_400_BAD_REQUEST)
        return self.get_sms_quota(request)


class SMSBuyView(APIView):
    permission_classes = (IsAdminUser, )

    def post(self, request):
        serializer = SMSBuySerializer(data=request.data)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        quantity = serializer.data["quantity"]
        to = [constance_config.EMAIL_SMS_CREDITREQUEST, ]
        subject = "Commande de %d SMS" % (quantity,)
        body = "Nouvelle commande de %d SMS demandée via CLARIWEB: %s" % (
            quantity, constance_config.CLARIWEB_ABSOLUTE_URL)
        send_mail(subject, body, constance_config.EMAIL_FROM, to,
                  fail_silently=False)
        return Response({"credit_requested": quantity},
                        status=status.HTTP_200_OK)


class LoginAsUserSelector(mixins.ListModelMixin,
                          viewsets.GenericViewSet):
    serializer_class = FullClariwebUserSerializer
    permission_classes = (IsAdminUser, )
    filter_backends = (SearchFilter, )
    search_fields = ('^user__username^', '^user__first_name^',
                     '^user__last_name^')
    pagination_class = LoginAsUserPagination

    def get_queryset(self):
        qs = ClariwebUser.objects.all()
        if not self.request.user.is_superuser:
            qs = ClariwebUser.objects.filter(user__is_superuser=False)
        # TODO: SQL Optimization
        # like .prefetch_related("email_address_set") (is_confirmed SQL storm)
        return qs


class AbstractUserView(object):
    def _getuser(self, request):
        response = None
        user = None

        return user, response


class UserModifyView(AbstractUserView, APIView):
    permission_classes = (IsAdminUser, )

    def post(self, request):
        serializer = UserModifySerializer(data=request.data)
        if not serializer.is_valid():
            response = Response({"errors": serializer.errors},
                                status=status.HTTP_400_BAD_REQUEST)
        else:
            try:
                user = BasicUser.objects.get(
                    username=serializer.data["username"])
            except BasicUser.DoesNotExist:
                return Response(
                    {"errors": {"general": "User does not exists."}},
                    status=status.HTTP_404_NOT_FOUND)
            if serializer.data.get("is_active") is not None:
                if user.is_staff:
                    return Response({
                        "errors": {"general": "Staff users can't be disabled."}
                    }, status=status.HTTP_400_BAD_REQUEST)
                user.is_active = serializer.data["is_active"]
            if serializer.data.get("password") is not None:
                user.set_password(serializer.data["password"])
                user.localpassword = True
            user.save(update_fields=('is_active', 'password',
                                     'localpassword'))
            serializer = FullClariwebUserSerializer(user.clariwebuser)
            response = Response(
                serializer.data,
                status=status.HTTP_200_OK)
        return response


class LoginAsUserView(AbstractUserView, APIView):
    # Yes: it's weird, but required because
    # want to access this view from a "normal" user
    # (only if was_staff is True!)
    permission_classes = (IsAuthenticated, )
    _signer = TimestampSigner()

    def login_as(self, request, user, store_original_user=True):
        original_user_pk = request.user.pk
        request.user = user
        request.session[SESSION_KEY] = user.id
        request.session[HASH_SESSION_KEY] = user.get_session_auth_hash()
        if store_original_user:
            request.session["login_as_userpk"] = self._signer.sign(
                original_user_pk)
        request.session.modified = True

    def restore_user(self, request, user):
        original_session = request.session.get("login_as_userpk")
        if not original_session:
            return
        try:
            original_user_pk = self._signer.unsign(
                original_session,
                max_age=datetime.timedelta(hours=1)
            )
        except SignatureExpired:
            return False
        user = BasicUser.objects.get(pk=original_user_pk)
        self.login_as(request, user, store_original_user=False)
        request.session.pop("login_as_userpk", None)
        return True

    def post(self, request):
        if (
            not request.user.is_staff and
            not request.session.get("login_as_userpk")
        ):
            logout(request)
            return Response({}, status=status.HTTP_403_FORBIDDEN)
        data = request.data
        serializer = UsernameSerializer(data=data)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        else:
            try:
                user = BasicUser.objects.get(
                    username=serializer.data["username"])
            except BasicUser.DoesNotExist:
                return Response({"errors": "User does not exists."},
                                status=status.HTTP_404_NOT_FOUND)
        if request.user.is_superuser:
            self.login_as(request, user)
        elif user.is_superuser:
            if not self.restore_user(request, user):
                logout(request)
                return Response({}, status=status.HTTP_403_FORBIDDEN)
        return Response({"username": user.username},
                        status=status.HTTP_200_OK)


class CustomSettingsView(APIView):
    permission_classes = (IsAdminUser, )

    def get(self, request):
        data = []
        for key, (default_value, help_text) in \
                constance_settings.CONFIG.items():
            valuetype = "text"
            # WARNING: Must be first.
            if "IP_ADDRESSES" in key:
                valuetype = "iplist"
            elif isinstance(default_value, bool):
                valuetype = "checkbox"
            elif isinstance(default_value, six.integer_types):
                valuetype = "number"
            if "PASSWORD" in key:
                valuetype = "password"
            if key == "CUSTOM_THEME_KEY":
                valuetype = "theme"
            data.append({
                "key": key,
                "type": valuetype,
                "help_text": help_text,
                "default_value": default_value,
                "value": getattr(constance_config, key)
            })
        return Response(data, status=status.HTTP_200_OK)

    def post(self, request):
        errors = []
        items = request.data.get("items", [])
        for item in items:
            key = item["key"]
            value = item["value"]
            if not key or key not in constance_settings.CONFIG.keys():
                errors.append("Invalid key %s." % (key,))
                continue
            default_value_type = type(constance_settings.CONFIG[key][0])
            if type(value) != default_value_type:
                errors.append("Invalid value type: %s != %s" %
                              (type(value), default_value_type))
                continue
            if "IP_ADDRESSES" in key:
                for netaddress in value:
                    try:
                        ipaddress.ip_network("{}".format(netaddress))
                    except ValueError:
                        errors.append(
                            "Network address %s is not valid" % (
                                netaddress,))
        if not errors:
            for item in items:
                setattr(constance_config, item["key"], item["value"])
        if errors or not items:
            return Response(errors, status=status.HTTP_400_BAD_REQUEST)
        else:
            return Response({}, status=status.HTTP_200_OK)
