from rest_framework import status
from rest_framework.views import APIView
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from brake.decorators import ratelimit
from clariweb.api.passwordreset.serializers import (
    AskSerializer, CheckSerializer, EditSerializer
)
from clariweb.clariweb.models import BasicUser, PasswordReset
from clariweb.clariweb.smscount import SMSCounter


class AskView(APIView):
    permission_classes = (AllowAny, )

    @staticmethod
    @ratelimit(rate='20/hour')
    def passwordreset_ask_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    def post(self, request):
        ratelimit = self.passwordreset_ask_ratelimit(request)
        if ratelimit is not None:
            return ratelimit
        resp_status = status.HTTP_400_BAD_REQUEST
        resp_data = {}

        if SMSCounter.get_quota_left() <= 0:
            return Response({}, status=status.HTTP_503_SERVICE_UNAVAILABLE)

        serializer = AskSerializer(data=request.data)
        if serializer.is_valid():
            resp_status = status.HTTP_404_NOT_FOUND
            users = BasicUser.objects.filter(
                first_name__unaccent__iexact=serializer.data["first_name"],
                last_name__unaccent__iexact=serializer.data["last_name"],
                clariwebuser__date_naissance=serializer.data["birthdate"])
            if users and len(users) == 1:
                user = users[0]
                clariwebuser = user.clariwebuser
                mobilephone = clariwebuser.address_mobile_phone
                if not mobilephone:
                    resp_status = status.HTTP_406_NOT_ACCEPTABLE
                    resp_data = {"errors": ["No mobile phone for this user."]}
                else:
                    PasswordReset.objects.create(user=user)
                    resp_status = status.HTTP_200_OK
                    resp_data = {
                        "mobilephone": "{}".format(mobilephone)[-3:],
                        "username": user.username,
                    }
        else:
            resp_data = {"errors": serializer.errors}
        return Response(resp_data, status=resp_status)


class CheckView(APIView):
    permission_classes = (AllowAny, )

    @staticmethod
    @ratelimit(rate='5/min')
    def passwordreset_check_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    def get_passwordreset(self, username, validation_code):
        try:
            return PasswordReset.objects.get(
                user__username=username,
                validation_code=validation_code)
        except PasswordReset.DoesNotExist:
            return None

    def post(self, request):
        ratelimit = self.passwordreset_check_ratelimit(request)
        if ratelimit is not None:
            return ratelimit
        resp_status = status.HTTP_400_BAD_REQUEST
        resp_data = {}
        serializer = CheckSerializer(data=request.data)
        if serializer.is_valid():
            resp_status = status.HTTP_404_NOT_FOUND
            pr = self.get_passwordreset(serializer.data["username"],
                                        serializer.data["validation_code"])
            if pr and pr.valid:
                resp_status = status.HTTP_200_OK
        else:
            resp_data = {"errors": serializer.errors}
        return Response(resp_data, status=resp_status)


class EditView(CheckView):
    permission_classes = (AllowAny, )
    check_serializer = EditSerializer

    @staticmethod
    @ratelimit(rate='5/min')
    def passwordreset_edit_ratelimit(request):
        if getattr(request, 'limited', False):
            return Response({}, status=status.HTTP_429_TOO_MANY_REQUESTS)
        return None

    def post(self, request):
        ratelimit = self.passwordreset_edit_ratelimit(request)
        if ratelimit is not None:
            return ratelimit
        resp_status = status.HTTP_400_BAD_REQUEST
        resp_data = {}
        serializer = EditSerializer(data=request.data)
        if serializer.is_valid():
            resp_status = status.HTTP_404_NOT_FOUND
            pr = self.get_passwordreset(serializer.data["username"],
                                        serializer.data["validation_code"])
            if pr.valid:
                user = pr.user
                user.set_password(serializer.data["password"])
                user.localpassword = True
                user.save()
                resp_status = status.HTTP_200_OK
        else:
            resp_data = {"errors": serializer.errors}
        return Response(resp_data, status=resp_status)
