# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime
import hashlib
import httpretty
from django.urls import reverse
from django.conf import settings
from rest_framework import status
from clariweb.api.test_api import EmptyCacheAPITestCase, BasicTestCase
from clariweb.clariweb.models import PaymentRequest, PaymentResponse


class PaymentRequestListViewTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        super().setUp()
        self.u1 = self.createClariwebUser(
            "pat1", "PAT", 1,
            date_naissance=datetime.date(1980, 4, 1),
            address_mobile_phone="+336000000000",
            debpt=0)
        self.u2 = self.createClariwebUser(
            "pat2", "PAT", 2,
            date_naissance=datetime.date(1950, 1, 1),
            email="pat2@user.fr",
            debpt=1000)
        self.u3 = self.createClariwebUser(
            "pat3", "PAT", 3, first_name="Émmanuel", last_name="àlarùe",
            date_naissance=datetime.date(1930, 1, 1),
            address_mobile_phone="+336000000000",
            email="user3@user.fr",
            debpt=5000)
        self.med1 = self.createClariwebUser("med1", "MED", 10)
        httpretty.enable()
        httpretty.register_uri(
            httpretty.POST, "%s/reglement" % (settings.CLARILAB_API_URL,),
            responses=[
                httpretty.Response(body="{}", status=200),
            ]
        )

    def tearDown(self):
        super().tearDown()
        httpretty.disable()
        httpretty.reset()

    def _test_login(self, username, shouldpass=True, **kwargs):
        data = {"username": username,
                "password": kwargs.get("password", username)}
        url = reverse("api-login")
        response = self.client.post(url, data)
        assert response.status_code == status.HTTP_200_OK
        self.client.credentials(HTTP_X_SESSION_ID=response.data["sessionid"])
        return response

    def _add_payment(self, user, **kw):
        payment_request = PaymentRequest(
            vads_amount=kw.get("amount", user.clariwebuser.debpt),
            user=user,
            vads_cust_id=user.pk,
            vads_cust_email=user.email if user.is_confirmed else '',
            vads_cust_title=user.clariwebuser.civilite,
            vads_cust_last_name=user.last_name,
            vads_cust_first_name=user.first_name,
            vads_capture_delay=0,
            vads_payment_config='SINGLE',
            vads_return_mode='GET',
            vads_validation_mode=0,
        )
        payment_request.save()
        return payment_request

    def _add_response(self, payment_request, success=False):
        payment_response = PaymentResponse(
            vads_amount=payment_request.vads_amount,
            vads_result="00" if success else "02",
            vads_trans_date=payment_request.vads_trans_date,
            vads_trans_id=payment_request.vads_trans_id,
            vads_site_id=payment_request.vads_site_id,
        )
        payment_response.save()
        payment_request.pzresponse = payment_response
        payment_request.save()
        return payment_response

    def test_unlogged(self):
        response = self.client.get(reverse("api-payment-request-list"))
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_empty(self):
        self._test_login("pat1")
        response = self.client.get(reverse("api-payment-request-list"))
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 0
        assert not response.data["results"]

    def test_no_response(self):
        self._add_payment(self.u2)
        self._add_payment(self.u3)
        self._test_login("pat2")
        response = self.client.get(reverse("api-payment-request-list"))
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 1
        p = response.data["results"][0]
        assert p["vads_amount"] == self.u2.clariwebuser.debpt
        assert p["status"] == "waiting"

    def test_response_failed(self):
        pr = self._add_payment(self.u2)
        self._add_response(pr, success=False)
        self._test_login("pat2")
        response = self.client.get(reverse("api-payment-request-list"))
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 1
        p = response.data["results"][0]
        assert p["vads_amount"] == self.u2.clariwebuser.debpt
        assert p["status"] == "refused"

    def test_response_success(self):
        pr = self._add_payment(self.u2)
        self._add_response(pr, success=True)
        self._test_login("pat2")
        response = self.client.get(reverse("api-payment-request-list"))
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 1
        p = response.data["results"][0]
        assert p["vads_amount"] == self.u2.clariwebuser.debpt
        assert p["status"] == "paid"

    def test_response_paginated(self):
        for i in range(80):
            pr = self._add_payment(self.u2, amount=i)
            self._add_response(pr, success=True)
        self._test_login("pat2")
        response = self.client.get(
            reverse("api-payment-request-list"),
            {"page": 1})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 80
        assert len(response.data["results"]) == 50

        response = self.client.get(
            reverse("api-payment-request-list"),
            {"page": 2})
        assert response.status_code == status.HTTP_200_OK
        assert response.data["count"] == 80
        assert len(response.data["results"]) == 30


class PaymentResponseListViewTestCase(EmptyCacheAPITestCase, BasicTestCase):
    def setUp(self):
        self.u1 = self.createClariwebUser(
            "pat1", "PAT", 1,
            date_naissance=datetime.date(1980, 4, 1),
            address_mobile_phone="+336000000000",
            debpt=3253)
        self.u2 = self.createClariwebUser(
            "pat2", "PAT", 2,
            date_naissance=datetime.date(1950, 1, 1),
            email="pat2@user.fr",
            debpt=1000)

    def _get_signature(self, data):
        fields = []
        for field in sorted(data.keys()):
            if field.startswith("vads_"):
                value = data[field]
                if value is None:
                    value = ''
                fields.append("{}".format(value))
        fields.append(settings.VADS_CERTIFICATE)
        return hashlib.sha1("+".join(fields).encode("utf-8")).hexdigest()

    def test_response_invalid_signature(self):
        resp_data = {
            "vads_cust_id": "{}".format(self.u1.clariwebuser.sil_pk),
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_site_id": "1",
            "vads_result": "00",
            "vads_amount": "3252",
            "signature": "invalid",
        }
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_403_FORBIDDEN
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 0

    def test_no_request(self):
        resp_data = {
            "vads_cust_id": "{}".format(self.u1.clariwebuser.sil_pk),
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_site_id": "1",
            "vads_result": "00",
            "vads_amount": "3252",
            "signature": "valid",
        }
        resp_data["signature"] = self._get_signature(resp_data)
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_404_NOT_FOUND
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 0

    def test_response_invalid_data(self):
        pr = PaymentRequest(vads_site_id=1,
                            vads_cust_id=self.u1.clariwebuser.sil_pk,
                            vads_trans_id="111111",
                            vads_trans_date="20150626080301",
                            vads_amount=3252,
                            user=self.u1)
        pr.save()
        resp_data = {
            "vads_cust_id": "abcd",
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_site_id": "1",
            "vads_result": "00",
            "vads_amount": "3252",
            "signature": "valid",
        }
        resp_data["signature"] = self._get_signature(resp_data)
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_400_BAD_REQUEST
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 0
        pr = PaymentRequest.objects.get(
            vads_trans_id=111111, vads_trans_date="20150626080301",
            vads_site_id=1)
        assert not pr.payment_successful

    def test_response_good(self):
        pr = PaymentRequest(vads_site_id=1,
                            vads_cust_id=self.u1.clariwebuser.sil_pk,
                            vads_trans_id="111111",
                            vads_trans_date="20150626080301",
                            vads_amount=3252,
                            user=self.u1)
        pr.save()
        resp_data = {
            "vads_cust_id": "{}".format(self.u1.clariwebuser.sil_pk),
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_trans_status": "AUTHORISED",
            "vads_site_id": "1",
            "vads_result": "00",
            "vads_amount": "3251",
            "vads_ctx_mode": "TEST",
            "vads_version": "V2",
            "signature": "valid",
        }
        resp_data["signature"] = self._get_signature(resp_data)
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_200_OK, r.data
        # We ask 1 cent less
        # We got 2 cent less, so our debpt should be 2 cents.
        assert r.data["debpt"] == 2
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 1
        pr = PaymentRequest.objects.get(
            vads_trans_id=111111, vads_trans_date="20150626080301",
            vads_site_id=1)
        assert pr.payment_successful

        # Now check an update
        resp_data = {
            "vads_cust_id": "{}".format(self.u1.clariwebuser.sil_pk),
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_trans_status": "AUTHORISED",
            "vads_site_id": "1",
            "vads_result": "00",
            "vads_amount": "3253",
            "vads_ctx_mode": "TEST",
            "vads_version": "V2",
            "signature": "valid",
        }
        resp_data["signature"] = self._get_signature(resp_data)
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_200_OK, r.data
        # Yes, we expect the debpt NOT to change
        # otherwise it will confuse CLARILAB.
        # This must never happen anyway.
        assert r.data["debpt"] == 2
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 1
        pr = PaymentRequest.objects.get(
            vads_trans_id=111111, vads_trans_date="20150626080301",
            vads_site_id=1)
        assert pr.payment_successful

    def test_response_refused(self):
        pr = PaymentRequest(vads_site_id=1,
                            vads_cust_id=self.u1.clariwebuser.sil_pk,
                            vads_trans_id="111111",
                            vads_trans_date="20150626080301",
                            vads_amount=3252,
                            user=self.u1)
        pr.save()
        resp_data = {
            "vads_cust_id": "{}".format(self.u1.clariwebuser.sil_pk),
            "vads_trans_date": "20150626080301",
            "vads_trans_id": "111111",
            "vads_trans_status": "REFUSED",
            "vads_site_id": "1",
            "vads_result": "02",
            "vads_amount": "3251",
            "vads_ctx_mode": "TEST",
            "vads_version": "V2",
            "signature": "valid",
        }
        resp_data["signature"] = self._get_signature(resp_data)
        r = self.client.post(reverse("api-payment-response"), data=resp_data)
        assert r.status_code == status.HTTP_200_OK, r.data
        # We ask 1 cent less
        # We got 2 cent less, so our debpt should be 2 cents.
        assert r.data["debpt"] == self.u1.clariwebuser.debpt
        assert PaymentResponse.objects.filter(
                vads_trans_id=111111, vads_trans_date="20150626080301",
                vads_site_id=1).count() == 1
        pr = PaymentRequest.objects.get(
            vads_trans_id=111111, vads_trans_date="20150626080301",
            vads_site_id=1)
        assert not pr.payment_successful
