# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from rest_framework import status, generics
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView
from clariweb.clariweb.models import PaymentRequest, PaymentResponse, Document
from clariweb.api.payment.serializers import (
    PaymentRequestSerializer, PaymentResponseSerializer,
    PaymentRequestListSerializer)
from clariweb.api.pagination import StandardPagination
from django_payzen import tools


class PaymentRequestView(APIView):
    def _get_return_uri(self, request, state):
        return "%s/%s" % (request.data["return_url"], state)

    def post(self, request):
        if not request.data.get("return_url"):
            return Response({'errors': u'Invalid request.'},
                            status=status.HTTP_400_BAD_REQUEST)
        user = self.request.user
        clariwebuser = request.user.clariwebuser
        if not clariwebuser:
            return Response({'errors': u'Erreur interne.'},
                            status=status.HTTP_404_NOT_FOUND)
        if clariwebuser.kind != "PAT":
            return Response(
                {'errors':
                    u'Paiement disponibles pour les patients seulements'},
                status=status.HTTP_404_NOT_FOUND)
        debpt = clariwebuser.debpt
        if debpt <= 0:
            return Response(
                {'errors': u'Aucune dette a payer'},
                status=status.HTTP_404_NOT_FOUND)
        payment_request = PaymentRequest(
            vads_amount=debpt,
            user=user,
            vads_cust_id=user.pk,
            vads_cust_email=user.email if user.is_confirmed else '',
            vads_cust_title=clariwebuser.civilite,
            vads_cust_last_name=user.last_name,
            vads_cust_first_name=user.first_name,
            vads_capture_delay=0,
            vads_language='fr',
            vads_payment_config='SINGLE',
            vads_return_mode='GET',
            vads_redirect_success_timeout=120,
            vads_redirect_error_timeout=120,
            vads_validation_mode=0,
            vads_contrib='clariweb',
            vads_url_success=self._get_return_uri(request, "success"),
            vads_url_referral=self._get_return_uri(request, "referral"),
            vads_url_refused=self._get_return_uri(request, "refused"),
            vads_url_cancel=self._get_return_uri(request, "cancel"),
            vads_url_error=self._get_return_uri(request, "error"),
            vads_url_return=self._get_return_uri(request, "return"),
        )
        payment_request.save(serializer=PaymentRequestSerializer())
        serializer = PaymentRequestSerializer(payment_request)
        data = serializer.data
        data["payzen_url"] = settings.PAYZEN_REQUEST_URL
        return Response(data, status=status.HTTP_200_OK)


class PaymentResponseView(APIView):
    permission_classes = (AllowAny, )

    def post(self, request):
        # Check validity of the signature
        signature_err_msg = ''
        try:
            if not tools.is_signature_valid(request.data):
                if settings.DEBUG:
                    signature_err_msg = 'Invalid signature'
                else:
                    signature_err_msg = 'Internal error'
        except TypeError:
            signature_err_msg = 'Internal error, invalid type'

        if signature_err_msg:
            return Response({'errors': [signature_err_msg, ]},
                            status=status.HTTP_403_FORBIDDEN)

        try:
            payment_request = PaymentRequest.objects.get(
                vads_trans_id=request.data.get("vads_trans_id"),
                vads_trans_date=request.data.get("vads_trans_date"),
                vads_site_id=request.data.get("vads_site_id"))
        except PaymentRequest.DoesNotExist:
            return Response(
                {'errors': ['Invalid response: no matching request found.']},
                status=status.HTTP_404_NOT_FOUND)

        # Sanitize values passed by PayZen (PositiveInteger='')
        sanitized_datas = {}
        for k, v in request.data.items():
            sanitized_datas[k] = v if v else None

        try:
            payment_response = PaymentResponse.objects.get(
                vads_trans_id=request.data.get("vads_trans_id"),
                vads_trans_date=request.data.get("vads_trans_date"),
                vads_site_id=request.data.get("vads_site_id"))
            serializer = PaymentResponseSerializer(payment_response,
                                                   data=sanitized_datas)
        except PaymentResponse.DoesNotExist:
            serializer = PaymentResponseSerializer(data=sanitized_datas)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        else:
            success = "error"
            payment_response = serializer.save()
            if not payment_request.pzresponse:
                payment_request.pzresponse = payment_response
                payment_request.save()
                if payment_response.payment_successful:
                    success = "success"
                    clariwebuser = payment_request.user.clariwebuser
                    clariwebuser.debpt -= payment_response.vads_amount
                    clariwebuser.save()
                    Document.objects \
                        .filter(user=clariwebuser, unpaid_amount__gt=0) \
                        .update(unpaid_amount=0)
            return Response({
                "success": success,
                "debpt": payment_request.user.clariwebuser.debpt
            }, status=status.HTTP_200_OK)


class PaymentRequestListView(generics.ListAPIView):
    serializer_class = PaymentRequestListSerializer
    pagination_class = StandardPagination

    def get_queryset(self):
        return PaymentRequest.objects.filter(user=self.request.user) \
            .order_by('-date_insert')
