# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import base64
import hashlib
import constance
from django.utils import timezone
from django.urls import reverse
from django.core.cache import cache
from django.conf import settings
from rest_framework import status
from rest_framework.test import APITestCase
from clariweb.clariweb.models import BasicUser, ClariwebUser, Document, Notification
from clariweb.clariweb.test_clariweb import ClariwebTestCaseBase


class EmptyCacheAPITestCase(APITestCase):
    def tearDown(self):
        super().tearDown()
        cache.clear()


class APIRPCTestCase(EmptyCacheAPITestCase):
    @staticmethod
    def _get_actor_dict(cwu):
        return {
            "nom": cwu.first_name,
            "prenom": cwu.last_name,
            "civilite": cwu.clariwebuser.civilite,
            "date_naissance": cwu.clariwebuser.date_naissance,
        }

    def setUp(self):
        super().setUp()
        self.a_med1 = self._get_actor_dict(self.med1)
        self.a_med2 = self._get_actor_dict(self.med2)
        self.a_med3 = self._get_actor_dict(self.med3)
        self.a_corr1 = self._get_actor_dict(self.corr1)
        self.a_corr2 = self._get_actor_dict(self.corr2)
        self.a_pat1 = self._get_actor_dict(self.pat1)
        self.a_pat2 = self._get_actor_dict(self.pat2)
        token = "Token %s" % settings.API_RPC_TOKEN
        self.client.credentials(
            HTTP_AUTHORIZATION=token)

    @staticmethod
    def getUserDict(id_sil, id_type, username, **kw):
        ud = {
            "id_utilisateur": id_sil,
            "type": id_type,
            "login": username,
            "password": kw.get("password", username),
            "personne": {
                "nom": kw.get("nom", username),
                "prenom": kw.get("prenom"),
                "civilite": kw.get("civilite"),
                "date_naissance": kw.get("date_naissance"),
                "numero_secu": kw.get("numero_secu"),
            },
            "adresse": {
                "ville": kw.get("ville"),
                "code_postal": kw.get("code_postal"),
                "ligne1": kw.get("ligne1"),
                "ligne2": kw.get("ligne2"),
                "ligne3": kw.get("ligne3"),
                "telephone": kw.get("telephone"),
                "gsm": kw.get("gsm"),
                "email": kw.get("email"),
            },
            "more": {
                "groupe": kw.get("groupe"),
                "groupe_clinique": kw.get("groupe_clinique"),
                "force_password": kw.get("force_password", False),
                "debpt": kw.get("debpt", 0),
                "demandes_unpaid": kw.get("demandes_unpaid", [])
            },
        }
        if kw.get("service_clinique"):
            ud["more"]["service_clinique"] = kw["service_clinique"]
        return ud

    def getDocDict(self, id_dest, id_doc, id_demande, **kw):
        fakeFile = b"this is a not a valid PDF file."
        file = kw.get("file", fakeFile)
        d = {
            "id_utilisateur": id_dest,
            "id_document": id_doc,
            "demande": {
                "id_demande": id_demande,
                "numero_demande": kw.get("numero_demande", str(id_demande)),
                "date_demande": self.get_date(
                    kw.get("date_demande", timezone.now())),
            },
            "file": {
                "type_document": kw.get("type_document",
                                        "Compte-Rendu PDF"),
                "content_type": kw.get("content_type", "application/pdf"),
                "url_dynamic_request": None,
                "static_file": base64.b64encode(file) if file else None,
                "invoice_id": kw.get("invoice_id"),
            },
            "more": {
                "a_poster": False,
                # 4 | 2 | 1 = SMS + VOCAL + email
                "type_notification": kw.get("notification", 1 | 4),
                "analyses": kw.get("analyses", []),
            },
        }
        if kw.get("medecin"):
            d["medecins"] = [{
                "nom": kw["medecin"].get("nom", ""),
                "prenom": kw["medecin"].get("prenom", ""),
                "civilite": kw["medecin"].get("civilite", ""),
            }]
        if kw.get("transmetteur"):
            d["transmetteur"] = {
                "nom": kw["transmetteur"].get("nom", ""),
                "prenom": kw["transmetteur"].get("prenom", ""),
                "civilite": kw["transmetteur"].get("civilite", ""),
            }
        if kw.get("patient"):
            d["patient"] = {
                "nom": kw["patient"].get("nom", ""),
                "prenom": kw["patient"].get("prenom", ""),
                "civilite": kw["patient"].get("civilite", ""),
                "date_naissance": kw["patient"].get("date_naissance", None),
            }
        if kw.get("clinic"):
            d["clinic"] = {
                "ipp": kw["clinic"].get("ipp"),
                "hospi": kw["clinic"].get("hospi"),
                "bed": kw["clinic"].get("bed"),
            }
        return d

    @staticmethod
    def get_date(d):
        return timezone.datetime.strftime(d, "%Y-%m-%dT%H:%M:%SZ")

    def add_utilisateur(self, userdict):
        url = reverse("api-rpc-user")
        r = self.client.post(url, userdict)
        return r.status_code == status.HTTP_200_OK

    def add_document(self, docdict):
        url = reverse("api-rpc-document")
        r = self.client.post(url, docdict)
        assert r.status_code == status.HTTP_200_OK, r.data
        return r.status_code == status.HTTP_200_OK


class RPCUserTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    PAT = 1
    MED = 2
    COR = 3

    def test_not_authed(self):
        url = reverse("api-rpc-user")
        self.client.credentials()
        response = self.client.post(url)
        assert response.status_code == status.HTTP_403_FORBIDDEN

    def test_badcall_user(self):
        url = reverse("api-rpc-user")
        response = self.client.post(url, {})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_badcall_document(self):
        url = reverse("api-rpc-document")
        response = self.client.post(url, {})
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        response = self.client.delete(url, {})
        assert response.status_code == status.HTTP_400_BAD_REQUEST

    def test_createuser(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             prenom="laurent")
        assert self.add_utilisateur(u)
        user = BasicUser.objects.get(username="patient1")
        assert user.last_name == "COUSTET"
        assert user.first_name == "laurent"

    def test_createuser_email(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="laurent@clarisys.fr")
        assert self.add_utilisateur(u)
        user = BasicUser.objects.get(username="patient1")
        assert user.email == "laurent@clarisys.fr"

    def test_createuser_invalidemail(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="W0")
        assert self.add_utilisateur(u)
        user = BasicUser.objects.get(username="patient1")
        assert user.email == ""

    def test_createuser_daylightsaving_birthdate(self):
        u = self.getUserDict(
            1, self.PAT, "patient1", nom="COUSTET",
            date_naissance=self.get_date(timezone.datetime(1976, 9, 26)))
        assert self.add_utilisateur(u)

    def test_modifyuser(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="laurent@clarisys.fr")
        assert self.add_utilisateur(u)
        u.update({
            "personne": {
                "nom": "COUSTET",
                "prenom": "Laurent",
                "civilite": "Mr",
                "date_naissance": self.get_date(timezone.datetime(1985, 5, 5))
            },
            "adresse": {
                "ville": "TOULOUSE",
                "code_postal": "31000",
                "ligne1": "1234 ligne très longue d'adresse avec de l'utf8",
                "ligne3": "1234 ligne très longue d'adresse avec de l'utf8",
                "telephone": " 05 05 05 05 05",
                "gsm": "+336 11.33 22 44",
                "email": "modified@clarisys.fr",
            },
            "more": {
                "groupe": "gr1",
                "groupe_clinique": "clinique2",
                "service_clinique": "URGENCE",
            }
        })
        assert self.add_utilisateur(u)
        dbuser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        clariwebuser = dbuser.clariwebuser
        assert dbuser.get_full_name() == "Mr Laurent COUSTET"
        assert clariwebuser.date_naissance == timezone.datetime(1985, 5, 5).date()
        assert clariwebuser.address_city == "TOULOUSE"
        assert clariwebuser.address_postal_code == "31000"
        assert clariwebuser.address_line1 == u["adresse"]["ligne1"]
        assert clariwebuser.address_line2 == ''
        assert clariwebuser.address_line3 == u["adresse"]["ligne3"]
        assert "{}".format(clariwebuser.address_phone) == "+33505050505"
        assert "{}".format(clariwebuser.address_mobile_phone) == "+33611332244"
        assert "{}".format(clariwebuser.group) == u["more"]["groupe"]
        assert "{}".format(clariwebuser.group_clinic) == u["more"]["groupe_clinique"]
        assert "{}".format(clariwebuser.service_clinic) == u["more"]["service_clinique"]
        assert dbuser.email == 'modified@clarisys.fr'

    def test_modifyuser_login(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        u.update({"login": "patient.modified1"})
        assert self.add_utilisateur(u)
        dbuser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        assert dbuser.username == u["login"]

    def test_modifyuser_datenaissance_empty(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="xx@clarisys.fr")
        u["personne"]["date_naissance"] = None
        assert self.add_utilisateur(u)
        cwu = ClariwebUser.objects.get(sil_pk=1)
        assert cwu.date_naissance is None
        u["personne"]["date_naissance"] = ""
        assert self.add_utilisateur(u)
        cwu = ClariwebUser.objects.get(sil_pk=1)
        assert cwu.date_naissance is None

    def test_modifyuser_invalidemail(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="xx@clarisys.fr")
        assert self.add_utilisateur(u)
        u.update({"adresse": {"email": "W0"}})
        assert self.add_utilisateur(u)
        dbuser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        assert dbuser.email == ''
        emails = dbuser.email_address_set.all()
        assert len(emails) == 0, [e.email for e in emails]

    def test_modifyuser_invalidemail_invalid_chars(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET",
                             email="xx@clarisys.fr")
        assert self.add_utilisateur(u)
        u.update({"adresse": {"email": "lau rent @ clarisys"}})
        assert self.add_utilisateur(u)
        dbuser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        assert dbuser.email == ''
        emails = dbuser.email_address_set.all()
        assert len(emails) == 0, [e.email for e in emails]

    def test_modifyuser_password(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        oldpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        u.update({"password": "testpassword"})
        assert self.add_utilisateur(u)
        newpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        assert oldpass != newpass

    def test_modifyuser_password_modifiedlocally(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        olduser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        olduser.localpassword = True
        olduser.save()
        oldpass = olduser.password
        u.update({"password": "testpassword"})
        assert self.add_utilisateur(u)
        newpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        assert oldpass == newpass

    def test_modifyuser_password_modifiedlocally_force(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        olduser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        olduser.localpassword = True
        olduser.save()
        oldpass = olduser.password
        u.update({"password": "testpassword",
                  "more": {"force_password": True}})
        assert self.add_utilisateur(u)
        newpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        assert newpass != oldpass

    def test_badphone(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        u.update({
            "adresse": {
                "telephone": "badnumber",
                "gsm": "241254",
            }
        })
        assert self.add_utilisateur(u)
        dbuser = BasicUser.objects.get(clariwebuser__sil_pk=1)
        clariwebuser = dbuser.clariwebuser
        assert not clariwebuser.address_phone
        assert not clariwebuser.address_mobile_phone

    def test_delete_medecin(self):
        """
        On test que la suppression d'un médecin deja associé à un document
        ne lève pas une IntegrityError
        """
        u_sil_pk = self.pat1.clariwebuser.sil_pk
        self.add_document(
            self.getDocDict(
                id_dest=u_sil_pk,
                patient=self.a_pat1,
                id_doc=1,
                id_demande=1,
                medecin=self.a_med1))
        assert Document.objects.get(sil_document_pk=1).medecin is not None
        self.add_document(
            self.getDocDict(
                id_dest=u_sil_pk,
                patient=self.a_pat1,
                id_doc=1,
                id_demande=1))
        assert Document.objects.get(sil_document_pk=1).medecin is None

    def test_modify_user_do_not_change_password(self):
        """
        Verifier que le mot de passe d'un utilisateur n'est pas mis à jour
        s'il n'a pas changé.
        """
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        assert self.add_utilisateur(u)
        oldpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        assert self.add_utilisateur(u)
        newpass = BasicUser.objects.get(clariwebuser__sil_pk=1).password
        assert oldpass == newpass


class RPCDebptTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    PAT = 1

    def test_patient_debpt(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        u["more"]["debpt"] = 300  # 3 euros equals 300 100th euros.
        assert self.add_utilisateur(u)
        clariwebuser = BasicUser.objects.get(
            clariwebuser__sil_pk=1).clariwebuser
        assert clariwebuser.debpt == 300

    def test_patient_debpt_zero(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        u["more"]["debpt"] = 0
        assert self.add_utilisateur(u)
        clariwebuser = BasicUser.objects.get(
            clariwebuser__sil_pk=1).clariwebuser
        assert clariwebuser.debpt == 0

    def test_patient_debpt_negative(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        u["more"]["debpt"] = -300
        assert self.add_utilisateur(u)

    def test_patient_debpt_unset(self):
        u = self.getUserDict(1, self.PAT, "patient1", nom="COUSTET")
        u["more"]["debpt"] = ''
        assert not self.add_utilisateur(u)

    def test_patient_debpt_multidemande(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = True
        u_sil_pk = self.pat1.clariwebuser.sil_pk
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=1, id_demande=1))
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=2, id_demande=2))
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=3, id_demande=2,
                            type_document="Note Honoraire"))
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=4, id_demande=3))
        Document.objects.filter(sil_document_pk=4).update(unpaid_amount=90000)
        u = self.getUserDict(u_sil_pk, self.PAT, self.pat1.first_name,
                             nom=self.pat1.last_name)
        u["more"]["debpt"] = 3000
        u["more"]["demandes_unpaid"] = [
            {"demande_id": 1, "amount": 1000},
            {"demande_id": 2, "amount": 2000},
        ]
        assert self.add_utilisateur(u)
        doc1 = Document.objects.get(sil_document_pk=1)
        doc2 = Document.objects.get(sil_document_pk=2)
        doc3 = Document.objects.get(sil_document_pk=3)
        doc4 = Document.objects.get(sil_document_pk=4)
        assert doc1.unpaid_amount == 1000
        assert doc2.unpaid_amount == 2000
        assert doc3.unpaid_amount == 2000
        assert doc4.unpaid_amount == 0

    def test_patient_demande_unpaid_negative(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = True
        u_sil_pk = self.pat1.clariwebuser.sil_pk
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=1, id_demande=1))
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=2, id_demande=2))
        u = self.getUserDict(u_sil_pk, self.PAT, self.pat1.first_name,
                             nom=self.pat1.last_name)
        u["more"]["debpt"] = -50
        u["more"]["demandes_unpaid"] = [
            {"demande_id": 1, "amount": 1000},
            {"demande_id": 2, "amount": -50},
        ]
        assert self.add_utilisateur(u)
        doc1 = Document.objects.get(sil_document_pk=1)
        doc2 = Document.objects.get(sil_document_pk=2)
        assert doc1.unpaid_amount == 1000
        assert doc2.unpaid_amount == 0
        assert doc1.user.debpt == 0

    def test_patient_debpt_anotheruser(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = True
        assert self.add_document(
            self.getDocDict(id_dest=self.pat1.clariwebuser.sil_pk,
                            patient=self.a_pat1,
                            id_doc=1, id_demande=1))
        assert self.add_document(
            self.getDocDict(id_dest=self.pat2.clariwebuser.sil_pk,
                            patient=self.a_pat2,
                            id_doc=2, id_demande=1))
        u = self.getUserDict(self.pat1.clariwebuser.sil_pk,
                             self.PAT, self.pat1.first_name,
                             nom=self.pat1.last_name)
        Document.objects.filter(sil_document_pk=2).update(unpaid_amount=9999)
        u["more"]["debpt"] = 3000
        u["more"]["demandes_unpaid"] = [
            {"demande_id": 1, "amount": 1000},
            {"demande_id": 2, "amount": 2000},
        ]
        assert self.add_utilisateur(u)
        doc1 = Document.objects.get(sil_document_pk=1,
                                    user=self.pat1.clariwebuser)
        doc2 = Document.objects.get(sil_document_pk=2,
                                    user=self.pat2.clariwebuser)
        assert doc1.unpaid_amount == 1000
        assert doc2.unpaid_amount == 9999

    def test_patient_debpt_payments_disabled(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = False
        assert self.add_document(
            self.getDocDict(id_dest=self.pat1.clariwebuser.sil_pk,
                            patient=self.a_pat1,
                            id_doc=1, id_demande=1))
        u = self.getUserDict(self.pat1.clariwebuser.sil_pk,
                             self.PAT, self.pat1.first_name,
                             nom=self.pat1.last_name)
        u["more"]["debpt"] = 3000
        u["more"]["demandes_unpaid"] = [
            {"demande_id": 1, "amount": 1000},
            {"demande_id": 2, "amount": 2000},
        ]
        assert self.add_utilisateur(u)
        doc1 = Document.objects.get(sil_document_pk=1,
                                    user=self.pat1.clariwebuser)
        assert doc1.unpaid_amount == 0


class RPCNotificationTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    def test_type_notification_sms(self):
        d = self.getDocDict(id_dest=30, patient=self.a_pat1,
                            id_doc=1, id_demande=1)
        d["more"]["type_notification"] = 1
        assert self.add_document(d)
        doc = Document.objects.get(
            user__sil_pk=30,
            sil_document_pk=1,
            sil_demande_pk=1
        )
        assert doc.notify_sms
        assert not doc.notify_email

    def test_type_notification_email(self):
        d = self.getDocDict(id_dest=30, patient=self.a_pat1,
                            id_doc=1, id_demande=1)
        d["more"]["type_notification"] = 4
        assert self.add_document(d)
        doc = Document.objects.get(
            user__sil_pk=30,
            sil_document_pk=1,
            sil_demande_pk=1
        )
        assert not doc.notify_sms
        assert doc.notify_email

    def test_type_notification_sms_and_email(self):
        d = self.getDocDict(id_dest=30, patient=self.a_pat1,
                            id_doc=1, id_demande=1)
        d["more"]["type_notification"] = 5
        assert self.add_document(d)
        doc = Document.objects.get(
            user__sil_pk=30,
            sil_document_pk=1,
            sil_demande_pk=1
        )
        assert doc.notify_sms
        assert doc.notify_email


class RPCDocumentTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    def test_add_document(self):
        d = self.getDocDict(id_dest=30, patient=self.a_pat1,
                            id_doc=1, id_demande=1)
        assert self.add_document(d)
        docs = Document.objects.filter(
            user__sil_pk=30,
            sil_document_pk=1,
            sil_demande_pk=1
        )
        assert len(docs) == 1

    def test_add_document_file(self):
        fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        with open(fpath, "rb") as f:
            d = self.getDocDict(id_dest=30, patient=self.a_pat1, id_doc=1,
                                id_demande=1, file=f.read())
            assert self.add_document(d)
            f.seek(0)
            orighash = hashlib.sha1(f.read()).hexdigest()
            doc = Document.objects.get(sil_document_pk=1)
            assert hashlib.sha1(doc.datafile.read()).hexdigest() == orighash
            assert not doc.is_invoice

    def test_add_document_facture(self):
        fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        with open(fpath, "rb") as f:
            d = self.getDocDict(id_dest=30, patient=self.a_pat1, id_doc=1,
                                id_demande=1, file=f.read())
            d["file"]["facture"] = True
            assert self.add_document(d)
            f.seek(0)
            dataorig = f.read()
            orighash = hashlib.sha1(dataorig).hexdigest()
            doc = Document.objects.get(sil_document_pk=1)
            doc.datafile.seek(0)
            dataread = doc.datafile.read()
            assert hashlib.sha1(dataread).hexdigest() == orighash
            assert doc.is_invoice

    def test_update_unpaid_amount(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = True
        fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        with open(fpath, "rb") as f:
            d = self.getDocDict(id_dest=30, patient=self.a_pat1, id_doc=1,
                                id_demande=1, file=f.read())
            d["file"]["facture"] = True
            d["more"]["debpt"] = 100
            assert self.add_document(d)
            doc = Document.objects.get(sil_document_pk=1)
            assert doc.unpaid_amount == 100
            del(d["more"]["debpt"])
            assert self.add_document(d)
            doc = Document.objects.get(sil_document_pk=1)
            assert doc.unpaid_amount == 0

    def test_amount_negative(self):
        constance.config.CUSTOM_PAYMENTS_ENABLED = True
        fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        with open(fpath, "rb") as f:
            d = self.getDocDict(id_dest=30, patient=self.a_pat1, id_doc=1,
                                id_demande=1, file=f.read())
            d["file"]["facture"] = True
            d["more"]["debpt"] = -50
            assert self.add_document(d)
            doc = Document.objects.get(sil_document_pk=1)
            assert doc.unpaid_amount == 0

    def test_document_modification_medecin(self):
        date_demande = timezone.make_aware(
            timezone.datetime(2014, 1, 1, 22, 22, 22),
            timezone.get_default_timezone())
        doc = self.createDocument(
            self.pat1, self.pat1.to_actor(), 1,
            medecin=self.med1.to_actor(),
            sil_demande_date=date_demande,
            document="Compte-Rendu PDF")
        assert doc.medecin.last_name == self.med1.last_name
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            medecin=self.a_med2,
            numero_demande=1,
            date_demande=date_demande)
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.medecin.last_name == self.med2.last_name

    def test_document_modification_transmetteur(self):
        date_demande = timezone.make_aware(
            timezone.datetime(2014, 1, 1, 22, 22, 22),
            timezone.get_default_timezone())
        doc = self.createDocument(
            self.pat1, self.pat1.to_actor(), 1,
            sil_demande_date=date_demande,
            document="Compte-Rendu PDF")
        assert doc.correspondant is None
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            transmetteur=self.a_corr1,
            numero_demande=1,
            date_demande=date_demande)
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.correspondant.last_name == self.corr1.last_name

    def test_document_overwrite(self):
        date_demande = timezone.make_aware(
            timezone.datetime(2014, 1, 1, 22, 22, 22),
            timezone.get_default_timezone())
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            medecin=self.a_med1,
            numero_demande=1,
            date_demande=date_demande)
        assert self.add_document(d)
        d["id_document"] = 5
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        # We need the old document value, not the new one.
        assert doc.sil_document_pk == 1
        assert doc.analyses == []

    def test_document_analyses_full(self):
        date_demande = timezone.make_aware(
            timezone.datetime(2014, 1, 1, 22, 22, 22),
            timezone.get_default_timezone())

        analyses = [
            {
                "code": "TP",
                "libelle": "Taux de Protombine",
                "anormaux": [],
                "a_saisir": False
            },
            {
                "code": "VS",
                "libelle": "Vitesse de sédimentation",
                "anormaux": [],
                "a_saisir": True
            },
            {
                "code": "NFS",
                "libelle": "Numération formule",
                "anormaux": [
                    "Leucocytes Supérieur au maximum",
                    "soit Inférieur au minimum"
                ],
                "a_saisir": False
            },
        ]
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            medecin=self.a_med1,
            numero_demande=1,
            date_demande=date_demande,
            analyses=analyses)
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        orig = d["more"]["analyses"]
        assert len(orig) == len(doc.analyses)
        for (idx, a) in enumerate(orig):
            assert a == doc.analyses[idx]

    def test_document_analyses_empty(self):
        date_demande = timezone.make_aware(
            timezone.datetime(2014, 1, 1, 22, 22, 22),
            timezone.get_default_timezone())
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            medecin=self.a_med1,
            numero_demande=1,
            date_demande=date_demande)
        del(d["more"]["analyses"])
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.analyses == []

    def test_document_invoice_id(self):
        d = self.getDocDict(id_dest=30, patient=self.a_pat1,
                            id_doc=1, id_demande=1,
                            type_document="Note Honoraire",
                            invoice_id="500231245")
        d["file"]["facture"] = True
        assert self.add_document(d)
        doc = Document.objects.get(sil_document_pk=1)
        assert doc.is_invoice
        assert doc.invoice_id == d["file"]["invoice_id"]

    def test_document_activation_key_static(self):
        """
        S"assurer que la clé d'activation ne change pas quand on ajoute un
        document pour un utilisateur dont l'email n'est pas encore confirmé.
        """
        u = self.pat1
        address = u.email_address_set.get(email=u.email)
        assert not address.is_confirmed
        u_sil_pk = u.clariwebuser.sil_pk
        assert self.add_document(
            self.getDocDict(id_dest=u_sil_pk,
                            patient=self.a_pat1,
                            id_doc=1, id_demande=1))
        address2 = u.email_address_set.get(email=u.email)
        assert address.key == address2.key

    def test_document_create_without_file(self):
        constance.config.EMAIL_NOTIFICATION_UNCONFIRMED = True
        d = self.getDocDict(id_dest=10, patient=self.a_pat1, id_doc=None,
                            id_demande=1, file=None)
        d["more"]["type_notification"] = 5
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.notify_sms
        assert doc.notify_email
        assert doc.available == False
        assert len(Notification.objects.all()) == 0

    def test_document_structured_no_datafile_overwrite(self):
        """
        1. Création du document sans fichier
        2. Mise à jour avec le fichier
        3. Mise à jour juste structuré
        """
        d = self.getDocDict(id_dest=20, patient=self.a_pat1, id_doc=None,
                            id_demande=1, file=None)
        d["more"]["structured_results"] = {"resultat": "structured"}
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.available == False
        assert doc.read_structured_results() == {"resultat": "structured"}
        assert not doc.datafile
        fpath = os.path.join(settings.BASE_DIR, "tests", "test1.pdf")
        with open(fpath, "rb") as f:
            dataorig = f.read()
            orighash = hashlib.sha1(dataorig).hexdigest()
            d2 = self.getDocDict(id_dest=20, patient=self.a_pat1, id_doc=None,
                                id_demande=1, file=dataorig)
            assert self.add_document(d2)
            doc = Document.objects.get(sil_demande_pk=1)
            doc.datafile.seek(0)
            dataread = doc.datafile.read()
            assert hashlib.sha1(dataread).hexdigest() == orighash
            assert doc.read_structured_results() == {"resultat": "structured"}
        d = self.getDocDict(id_dest=20, patient=self.a_pat1, id_doc=None,
                            id_demande=1, file=None)
        d["more"]["structured_results"] = {"resultat": "structured2"}
        assert self.add_document(d)
        doc = Document.objects.get(sil_demande_pk=1)
        assert doc.datafile
        assert doc.read_structured_results() == {"resultat": "structured2"}

    def test_document_structured_not_correspondant(self):
        """Seuls les correpondants peuvent recevoir des résultats structurés"""
        d = self.getDocDict(id_dest=30, patient=self.a_pat1, id_doc=None,
                            id_demande=1, file=None)
        d["more"]["structured_results"] = {"resultat": "structured"}
        r = self.client.post(reverse("api-rpc-document"), d)
        assert r.status_code == status.HTTP_400_BAD_REQUEST


class RPCClinicTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    def test_add_document(self):
        d = self.getDocDict(
            id_dest=30, patient=self.a_pat1, id_doc=1, id_demande=1,
            clinic={
                'hospi': 'AABBCC12512525',
                'ipp': '123451234AA',
                'bed': 'L32',
            }
        )
        assert self.add_document(d)
        doc = Document.objects.get(
            user__sil_pk=30,
            sil_document_pk=1,
            sil_demande_pk=1
        )
        assert doc
        assert doc.clinic_ipp == '123451234AA'
        assert doc.clinic_hospi == 'AABBCC12512525'
        assert doc.clinic_bed == 'L32'
        del(d["clinic"])
        assert self.add_document(d)
        doc = Document.objects.get(pk=doc.pk)
        assert doc.clinic_ipp == '123451234AA'
        assert doc.clinic_hospi == 'AABBCC12512525'
        assert doc.clinic_bed == 'L32'


class RPCUserMergeTestCase(ClariwebTestCaseBase, APIRPCTestCase):
    def test_merge_badcall(self):
        url = reverse("api-rpc-user-merge")
        r = self.client.post(url, {})
        assert r.status_code == status.HTTP_400_BAD_REQUEST

    def test_merge_bad_source_user(self):
        url = reverse("api-rpc-user-merge")
        r = self.client.post(url, {
            "id_utilisateur_dest": self.pat1.clariwebuser.sil_pk,
            "id_utilisateurs_source": [self.pat2.clariwebuser.sil_pk, 120598],
        })
        assert r.status_code == status.HTTP_404_NOT_FOUND

    def test_merge_bad_dest_user(self):
        url = reverse("api-rpc-user-merge")
        r = self.client.post(url, {
            "id_utilisateur_dest": 498124,
            "id_utilisateurs_source": [self.pat1.clariwebuser.sil_pk, ],
        })
        assert r.status_code == status.HTTP_404_NOT_FOUND

    def test_merge_singleuser(self):
        url = reverse("api-rpc-user-merge")
        r = self.client.post(url, {
            "id_utilisateur_dest": self.pat1.clariwebuser.sil_pk,
            "id_utilisateurs_source": [self.pat2.clariwebuser.sil_pk, ],
        })
        assert r.status_code == status.HTTP_200_OK, r
        qs = BasicUser.objects.filter(username=self.pat2.username)
        assert qs.count() == 0
