# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import json
import logging
import traceback
import constance
from rest_framework import status
from rest_framework.response import Response
from rest_framework.views import APIView
from django.core.files.uploadedfile import SimpleUploadedFile
from django.contrib.auth import get_user_model
from django.db.models import Q
from clariweb.api.rpc.serializers import (
    UserAddSerializer,
    DocumentSerializer,
    DocumentWriteSerializer,
    DocumentWriteSerializerStructured,
    UserMergeSerializer
)
from clariweb.api.rpc.forms import UtilisateurForm
from clariweb.api.rpc.permission import TokenPermission
from clariweb.clariweb.models import Document, Actor

logger = logging.getLogger(__name__)


class UserView(APIView):
    permission_classes = (TokenPermission, )

    def post(self, request, format=None):
        serializer = UserAddSerializer(data=request.data)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        user = None
        usermodel = get_user_model()
        personne = serializer.validated_data["personne"]
        adresse = serializer.validated_data["adresse"]
        more = serializer.validated_data["more"]
        try:
            id_utilisateur = serializer.validated_data["id_utilisateur"]
            user = usermodel.objects.get(
                clariwebuser__sil_pk=id_utilisateur)
            user.username = serializer.validated_data["login"]
            # We don't want the user to have his password changed by external
            # systems if it was changed locally in clariweb.
            if (
                (not user.localpassword or
                    more.get("force_password", False)) and
                not user.check_password(serializer.validated_data["password"])
            ):
                user.set_password(serializer.validated_data["password"])
            user.email = adresse.get("email", user.email) or ''
            user.last_name = personne.get("nom", user.last_name) or ''
            user.first_name = personne.get("prenom", user.first_name) or ''
            user.save()
        except usermodel.DoesNotExist:
            try:
                user = usermodel.objects.create_user(
                    username=serializer.validated_data["login"],
                    password=serializer.validated_data["password"],
                    first_name=personne.get("prenom"),
                    last_name=personne.get("nom"),
                    email=adresse.get("email"))
            except:
                logger.exception("User creation failed.")
                return Response(
                    traceback.format_exc(),
                    status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        form_dict = {
            "sil_pk": id_utilisateur,
            "kind": serializer.validated_data["type"],
            "civilite": personne.get("civilite"),
            "date_naissance": personne.get("date_naissance"),
            "numero_secu": personne.get("numero_secu"),
            "address_city": adresse.get("ville"),
            "address_postal_code": adresse.get("code_postal"),
            "address_line1": adresse.get("ligne1"),
            "address_line2": adresse.get("ligne2"),
            "address_line3": adresse.get("ligne3"),
            "address_phone": adresse.get("telephone"),
            "address_mobile_phone": adresse.get("gsm"),
            "group": more.get("groupe"),
            "group_clinic": more.get("groupe_clinique"),
            "service_clinic": more.get("service_clinique"),
            "debpt": more.get("debpt", 0),
        }
        # Mise à zero des dettes negatives avant intégration dans clariweb
        if form_dict["debpt"] < 0:
            form_dict["debpt"] = 0
        form = UtilisateurForm(form_dict, instance=user.clariwebuser)
        if form.is_valid():
            i = form.save()
            user.clariwebuser = i
            user.save()
            if constance.config.CUSTOM_PAYMENTS_ENABLED:
                demandes_unpaid = more.get("demandes_unpaid", [])
                # Mise à zero de tout montant impayé sur
                # les documents excepté ceux notés dans demandes_unpaid
                Document.objects \
                    .filter(user=user.clariwebuser, unpaid_amount__gt=0) \
                    .exclude(sil_demande_pk__in=[
                        d["demande_id"] for d in demandes_unpaid]) \
                    .update(unpaid_amount=0)
                for unpaid_demande in demandes_unpaid:
                    if unpaid_demande["amount"] < 0:
                        amount = 0
                    else:
                        amount = unpaid_demande["amount"]
                    Document.objects.filter(
                        user=user.clariwebuser,
                        sil_demande_pk=unpaid_demande["demande_id"]) \
                        .update(unpaid_amount=amount)
        else:
            logger.warning(
                "User creation validation error: %s",
                form.errors.as_text())
            return Response(
                form.errors.as_text(),
                status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        return Response('OK', status=status.HTTP_200_OK)


class DocumentView(APIView):
    permission_classes = (TokenPermission, )

    @staticmethod
    def _getClariwebUser(id_user):
        if id_user is None:
            return None
        usermodel = get_user_model()
        try:
            user = usermodel.objects.get(clariwebuser__sil_pk=id_user)
            return user.clariwebuser
        except usermodel.DoesNotExist:
            return None

    @staticmethod
    def _updateActor(a, data):
        if data:
            if not a:
                a = Actor()
            a.last_name = data["nom"]
            a.first_name = data.get("prenom", "")
            a.date_naissance = data.get("date_naissance")
            a.civilite = data.get("civilite", "")
            a.save()
            return a.pk
        else:
            if a:
                a.delete()
        return None

    def post(self, request, format=None):
        serializer = DocumentSerializer(data=request.data)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        id_utilisateur = serializer.validated_data["id_utilisateur"]
        patient = serializer.validated_data["patient"]
        try:
            medecin = serializer.validated_data.get("medecins", [])[0]
        except IndexError:
            medecin = None
        correspondant = serializer.validated_data.get("transmetteur")
        idemande = serializer.validated_data["demande"]
        ifile = serializer.validated_data["file"]
        imore = serializer.validated_data["more"]
        iclinic = serializer.validated_data.get("clinic")
        id_document = serializer.validated_data.get("id_document", None)

        # Try to resolve id_destinataire user
        user = self._getClariwebUser(id_utilisateur)
        user_pk = user.pk if user else None
        if not user_pk:
            return Response(
                "L'utilisateur id=%d n'existe pas." % id_utilisateur,
                status=status.HTTP_404_NOT_FOUND)
        if not ifile.get("static_file"):
            available = False
        else:
            available = True

        if imore.get("structured_results") and user.kind != "COR":
            return Response(
                "Seuls les correspondants peuvent recevoir des résultats structurés.",
                status=status.HTTP_400_BAD_REQUEST)

        # Essaye de trouver un document déja existant.
        # L'unicité du document est garantie de deux manières:
        #   - sil_document_pk
        # et
        #   - sil_demande_pk, sil_demande_date, user, document
        # La conséquence: il ne peut y avoir dans clariweb qu'un document
        # par type de document / demande / utilisateur.
        # Exemple: 1 compte-Rendu pour le patient demande "XXX".
        document = None
        demande_date = idemande.get("date_demande")
        documents = Document.objects.filter(
            Q(sil_demande_pk=idemande.get("id_demande")) &
            Q(sil_demande_date=demande_date) &
            Q(user=user_pk) &
            Q(document=ifile.get("type_document"))
        )[:1]
        if len(documents) == 1:  # Evaluate the queryset here
            document = documents[0]
            id_document = document.sil_document_pk

        form_dict = {
            "sil_demande_pk": idemande.get("id_demande"),
            "sil_demande_number": idemande.get("numero_demande"),
            "sil_demande_date": demande_date,
            "sil_document_pk": id_document,
            "document": ifile.get("type_document"),
            "content_type": ifile.get("content_type"),
            "user": user_pk,
            "medecin": self._updateActor(
                document.medecin if document else None, medecin),
            "patient": self._updateActor(
                document.patient if document else None, patient),
            "correspondant": self._updateActor(
                document.correspondant if document else None, correspondant),
            "is_invoice": ifile.get("facture", False),
            "invoice_id": ifile.get("invoice_id"),
            "notify_sms": ((imore.get("type_notification", 0) & 1) != 0),
            "notify_email": ((imore.get("type_notification", 0) & 4) != 0),
            "datafile": SimpleUploadedFile(
                "tempfile", ifile["static_file"], ifile.get("content_type")
            ) if ifile.get("static_file") else None,
            "analyses": imore.get("analyses", []),
            "unpaid_amount": imore.get("debpt", 0),
            "available": available,
        }
        if iclinic:
            form_dict['clinic_ipp'] = iclinic.get("ipp")
            form_dict['clinic_hospi'] = iclinic.get("hospi")
            form_dict['clinic_bed'] = iclinic.get("bed")
        # Mise à zero des dettes négatives avant integration dans clariweb
        if form_dict["unpaid_amount"] < 0:
            form_dict["unpaid_amount"] = 0
        user = get_user_model().objects.get(clariwebuser__pk=user_pk)
        if (
            not constance.config.CUSTOM_PAYMENTS_ENABLED or
            user.clariwebuser.kind != "PAT"
        ):
            form_dict["unpaid_amount"] = 0
        if imore.get("structured_results"):
            form_dict["structured_results"] = SimpleUploadedFile(
                "tmpjson", json.dumps(
                    imore["structured_results"],
                    ensure_ascii=False).encode("utf-8"),
                "application/json")
            writeSerializer = DocumentWriteSerializerStructured(
                document, data=form_dict)
        else:
            writeSerializer = DocumentWriteSerializer(
                document, data=form_dict)
        if writeSerializer.is_valid():
            writeSerializer.save()
        else:
            logger.warning(
                "Document creation validation error: %s",
                "{}".format(writeSerializer.errors))
            return Response("{}".format(writeSerializer.errors),
                            status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        return Response('OK', status=status.HTTP_200_OK)

    def delete(self, request, format=None):
        return Response('Not implemented.',
                        status=status.HTTP_400_BAD_REQUEST)


class UserMergeView(APIView):
    permission_classes = (TokenPermission, )

    @staticmethod
    def _getUser(id_user):
        if id_user is None:
            return None
        return get_user_model().objects.get(clariwebuser__sil_pk=id_user)

    def post(self, request, format=None):
        usermodel = get_user_model()
        serializer = UserMergeSerializer(data=request.data)
        if not serializer.is_valid():
            return Response({"errors": serializer.errors},
                            status=status.HTTP_400_BAD_REQUEST)
        id_user_dest = serializer.validated_data["id_utilisateur_dest"]
        id_users_source = serializer.validated_data["id_utilisateurs_source"]
        try:
            user_dest = self._getUser(id_user_dest)
        except usermodel.DoesNotExist:
            return Response("User %d does not exists." % id_user_dest,
                            status=status.HTTP_404_NOT_FOUND)
        try:
            users_source = [
                self._getUser(id_user) for id_user in id_users_source]
        except usermodel.DoesNotExist:
            return Response("One or more user in %s does not exists." % id_users_source,
                            status=status.HTTP_404_NOT_FOUND)
        try:
            user_dest.merge_foreignusers(users_source)
        except:
            return Response(traceback.format_exc(),
                            status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        return Response('OK', status=status.HTTP_200_OK)
