# -*- coding: utf-8 -*-

from constance import config as constance_config
from django.conf import settings
from suds import WebFault
from suds.client import Client
from suds.xsd.doctor import ImportDoctor, Import
import logging
import requests
from decimal import Decimal


wsdl_url = "https://www.ovh.com/soapi/soapi-re-1.61.wsdl"
# Fix missing types with ImportDoctor
schema_url = 'http://schemas.xmlsoap.org/soap/encoding/'
logger = logging.getLogger("backend.sms.ovh")


class SMSSendException(Exception):
    pass


class RetryException(SMSSendException):
    pass


class AbortException(SMSSendException):
    pass


def send_sms_http2sms(numbers, message, **kwargs):
    if not isinstance(numbers, (list, tuple)):
        numbers = [numbers]
    if not settings.TEST_MODE:
        assert(constance_config.SMS_LOGIN and
               constance_config.SMS_PASSWORD and
               constance_config.SMS_ACCOUNT and
               settings.SMS_OVH_HTTP2SMS_FROM)
    params = {
        "account": constance_config.SMS_ACCOUNT,
        "login": constance_config.SMS_LOGIN,
        "password": constance_config.SMS_PASSWORD,
        "from": kwargs.get("from", settings.SMS_OVH_HTTP2SMS_FROM),
        "to": ",".join(numbers),
        "message": message[:160],  # Cut the string
        "class": 1,
        "contentType": "application/json"
    }
    if settings.TEST_MODE:
        logger.debug("SMS -> %s: %s", numbers, message)
        logger.info("Test mode, no SMS sent.")
        return True
    logger.debug("Tryint to send SMS on %s: %s",
                 settings.SMS_OVH_HTTP2SMS, params)
    r = requests.get(settings.SMS_OVH_HTTP2SMS, params=params)
    # Real responses:
    # 200 {u'status': 100, u'creditLeft': u'983.00', u'smsIds': [u'14787947']}
    # 200 {u'status': 202, u'message':
    #      u'Following receivers phone number are invalid : "blah"'}
    if r.status_code != requests.codes.ok:
        # C'est un cas d'erreur ou il faut qu'on re essaye
        # l'api OVH est down...
        raise RetryException(r.status_code)
    smsIds = []
    data = r.json()
    # Code 200 chez OVH ça veut pas dire que c'est bon,
    # il faut checker si smsIds est présent.
    if not data.get("smsIds"):
        logger.warning("Failed to send SMS to %s: %s",
                       numbers, data.get("message"))
    else:
        logger.info("SMS credit left: %s", data.get("creditLeft"))
        logger.debug("sms sent to %s: %s", numbers, smsIds)
        smsIds = data.get("smsIds", [])
    return len(smsIds) > 0


def _suds_get_client():
    schema_import = Import(schema_url)
    schema_doctor = ImportDoctor(schema_import)

    client = Client(url=wsdl_url, doctor=schema_doctor)
    cache = client.options.cache
    cache.setduration(days=30)
    return client


def send_sms_soapi(numbers, message, **kwargs):
    if not isinstance(numbers, (list, tuple)):
        numbers = [numbers]
    if not settings.TEST_MODE:
        assert(constance_config.SMS_LOGIN and
               constance_config.SMS_PASSWORD and
               constance_config.SMS_ACCOUNT and
               settings.SMS_OVH_HTTP2SMS_FROM)
    if settings.TEST_MODE:
        logger.debug("SMS -> %s: %s", numbers, message)
        logger.info("Test mode, no SMS sent.")
        return True
    client = _suds_get_client()
    # int[] telephonySmsUserMultiSend ( stringlogin, stringpassword,
    # stringsmsAccount, stringnumberFrom, string[] numberTo,
    # stringmessage, intsmsValidity, intsmsClass, intsmsDeferred,
    # intsmsPriority, intsmsCoding, stringtag, stringnumberToCsv,
    # booleannoStop )
    # http://www.ovh.com/soapi/fr/?method=telephonySmsUserMultiSend
    sms_ids = []
    try:
        logger.debug("Trying to send SMS to %s using SOAPI", numbers)
        sms_ids = client.service.telephonySmsUserMultiSend(
            constance_config.SMS_LOGIN,
            constance_config.SMS_PASSWORD,
            constance_config.SMS_ACCOUNT,
            kwargs.get("from", settings.SMS_OVH_HTTP2SMS_FROM),
            list(numbers),
            message[:160],  # Cut the string
            2880,  # 48 hours validity
            1,  # Phone display
            0,  # No delay
            3,  # Priority
            2,  # Unicode encoding
            '',  # Tag
            '',  # CSV ?
            True)  # NoStop
        logger.info("SMS Sent: %s", sms_ids)
    except WebFault as e:
        logger.exception("Error trying to send SMS")
        raise RetryException(str(e))
    return len(sms_ids) > 0


def get_quota_left():
    client = _suds_get_client()
    quotaLeft = 0
    try:
        resp = client.service.telephonySmsGetUserQuotaForUser(
            constance_config.SMS_ACCOUNT,
            constance_config.SMS_LOGIN,
            constance_config.SMS_PASSWORD)
        quotaLeft = resp.quotaLeft
    except WebFault as e:
        logger.exception("Error de récupération du quota SMS")
        raise RetryException(str(e))
    return int(Decimal(quotaLeft).to_integral_value())


def handle_sms_callback():
    """
    Response from OVH callback:
    213.186.33.22 "GET /?&&id=14787922&ptt=3&dlr=4&
        date=2013-10-08%2017%3A39%3A02&descriptionDlr=Waiting
    213.186.33.22 "GET /?&&id=14787922&ptt=4&dlr=1&
        date=2013-10-08%2017%3A40%3A02&descriptionDlr=Success
    213.186.33.22 "GET /?&&id=14787947&ptt=4&dlr=1&
        date=2013-10-08%2017%3A41%3A02&descriptionDlr=Success
    213.186.33.22 "GET /?&&id=14787947&ptt=4&dlr=1&
        date=2013-10-08%2017%3A41%3A02&descriptionDlr=Success
    """
    pass


send_sms = send_sms_soapi
