# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import re
import json
import random
import logging
import datetime
import constance
from django.utils.timezone import utc
from django.db import models
from django.db.models import Q
from django.db.models.fields.files import FieldFile
from django.utils import timezone
from django.core.files.storage import FileSystemStorage
from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.core import validators
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import BaseUserManager
from django.contrib.postgres.fields import JSONField
from django.template.loader import get_template
from django.core.mail import send_mail
from simple_email_confirmation.models import SimpleEmailConfirmationUserMixin
from phonenumber_field.modelfields import PhoneNumberField
from django_payzen import tools
from django_payzen import models as payzen_models
from .storage import OverwriteFileSystemStorage, GZFileSystemStorage

structuredfs = GZFileSystemStorage(location=settings.DOCUMENTS_ROOT)
documentfs = OverwriteFileSystemStorage(location=settings.DOCUMENTS_ROOT)
logger = logging.getLogger("clariweb.models")


class ClariwebUserManager(BaseUserManager):
    def _create_user(self, username, password,
                     is_staff, is_superuser, **extra_fields):
        now = timezone.now()
        if not username:
            raise ValueError('The given username must be set')
        extra_fields["email"] = self.normalize_email(
            extra_fields.get("email", ''))
        extra_fields["first_name"] = extra_fields.get("first_name", "") or ''
        user = self.model(username=username,
                          is_staff=is_staff, is_active=True,
                          is_superuser=is_superuser, last_login=now,
                          date_joined=now, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, username, password=None, **extra_fields):
        return self._create_user(username, password, False, False,
                                 **extra_fields)

    def create_superuser(self, username, password, **extra_fields):
        return self._create_user(username, password, True, True,
                                 **extra_fields)


class BasicUser(AbstractBaseUser,
                SimpleEmailConfirmationUserMixin,
                PermissionsMixin):
    class Meta:
        verbose_name = "utilisateur"

    objects = ClariwebUserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = []

    username = models.CharField(
        _('username'), max_length=128, unique=True,
        help_text=_('Required. 128 characters or fewer. Letters, numbers and '
                    '@/./+/-/_ characters'),
        validators=[
            validators.RegexValidator(re.compile('^[\w.@+-]+$'),
                                      _('Enter a valid username.'),
                                      'invalid')
        ])
    email = models.EmailField(_('email address'), max_length=256, blank=True,
                              db_index=True)
    first_name = models.CharField(_('first name'), max_length=256, blank=True)
    last_name = models.CharField(_('last name'), max_length=256, blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)
    clariwebuser = models.OneToOneField('clariweb.ClariwebUser',
                                        blank=True, null=True, db_index=True,
                                        related_name='user',
                                        on_delete=models.CASCADE)
    localpassword = models.BooleanField(default=False)

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        if self.clariwebuser and self.clariwebuser.civilite:
            full_name = '%s %s' % (self.clariwebuser.civilite, full_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject.replace('\n', ''), message, from_email, [self.email])
        self.email_address_set.filter(email=self.email).update(email_sent=True)

    def send_email_confirmation(self):
        template_subject = get_template("email/email_confirmation_subject.txt")
        if self.clariwebuser.kind == "PAT":
            template_body = get_template("email/email_confirmation.txt")
        else:
            template_body = get_template("email/email_confirmation_autres.txt")
        context = {
            "user": self,
            "customization": constance.config
        }
        self.email_user(template_subject.render(context),
                        template_body.render(context),
                        from_email=constance.config.EMAIL_FROM)

    def is_using_session_login(self):
        if not constance.config.LOGIN_USING_SESSIONS:
            return False
        if self.is_staff:
            return True
        return self.clariwebuser and self.clariwebuser.kind in (
            'MED', 'COR', 'PREL')

    def is_underaged(self):
        if self.clariwebuser and self.clariwebuser.date_naissance:
            date_naissance = self.clariwebuser.date_naissance
            underage_delta = datetime.timedelta(settings.UNDERAGE * 365.15)
            age = (timezone.now().date() - date_naissance)
            return (age < underage_delta)
        return False

    def merge_foreignusers(self, src_users):
        # Check unicity of the destination merge list
        assert(len(set([u.pk for u in src_users])) == len(src_users))
        dst_cwuser = self.clariwebuser
        for src_user in src_users:
            src_cwuser = src_user.clariwebuser
            assert(src_cwuser != dst_cwuser)
            assert(src_cwuser.kind == dst_cwuser.kind)
            logger.info("Merging user %s into %s." % (src_cwuser, dst_cwuser))
            # Merge documents
            src_cwuser.documents_user.select_for_update() \
                .update(user=dst_cwuser)
            # Merge AccessLog
            DocumentAccessLog.objects.select_for_update() \
                .filter(user=src_cwuser).update(user=dst_cwuser)
            # Merge Notification
            Notification.objects.select_for_update().filter(user=src_cwuser) \
                .update(user=dst_cwuser)
            # Merge AccessLog
            AccessLog.objects.select_for_update().filter(user=src_user) \
                .update(user=self)
            # Remove PasswordReset
            PasswordReset.objects.select_for_update().filter(user=src_user) \
                .delete()
            # Merge debpt
            dst_cwuser.debpt += src_cwuser.debpt
            dst_cwuser.save(update_fields=("debpt", ))
            # Remove old user
            src_cwuser.delete()
            src_user.delete()

    @property
    def userdir(self):
        return "user_%d" % (self.pk,)

    def to_actor(self):
        """Only for testing purpose"""
        actor = Actor(last_name=self.last_name,
                      first_name=self.first_name,
                      civilite=self.clariwebuser.civilite,
                      date_naissance=self.clariwebuser.date_naissance)
        actor.save()
        return actor


class ClariwebUser(models.Model):
    USER_KIND_CHOICES = (
        ('MED', u'Médecin'),
        ('PAT', u'Patient'),
        ('COR', u'Correspondant'),
        ('PREL', u'Préleveur'),
    )

    sil_pk = models.BigIntegerField(u'id_sil', unique=True, db_index=True)
    kind = models.CharField(max_length=32, choices=USER_KIND_CHOICES)
    group = models.CharField(max_length=1024, blank=True, db_index=True)
    group_clinic = models.CharField(max_length=1024, blank=True, db_index=True)
    service_clinic = models.CharField(max_length=1024, blank=True)
    civilite = models.CharField(max_length=128, blank=True)
    date_naissance = models.DateField(blank=True, null=True)
    numero_secu = models.CharField(u'numéro de sécurité sociale',
                                   max_length=32, blank=True,
                                   db_index=True)
    address_city = models.CharField(max_length=256, blank=True)
    address_country = models.CharField(max_length=128, blank=True)
    address_postal_code = models.CharField(max_length=16, blank=True)
    address_line1 = models.CharField(max_length=256, blank=True)
    address_line2 = models.CharField(max_length=256, blank=True)
    address_line3 = models.CharField(max_length=256, blank=True)
    address_phone = PhoneNumberField(blank=True, null=True)
    address_mobile_phone = PhoneNumberField(blank=True, null=True)
    date_creation = models.DateField(auto_now_add=True)
    debpt = models.IntegerField("Dette en centimes d'euros", default=0)

    class Meta:
        ordering = ['-pk']

    def __str__(self):
        return "<ClariwebUser[%s]: %d (sil=%d) tied to %s>" % (
            self.kind, self.pk, self.sil_pk, self.user)


class DocumentManager(models.Manager):
    def foruser(self, user):
        document_qs = self.get_queryset()
        clariwebuser = user.clariwebuser
        if not user.is_superuser:
            if not clariwebuser:
                return []
            document_id_valuelist = DocumentPerUser.objects.filter(user=clariwebuser) \
                .values_list("document_id", flat=True)
            document_qs = Document.objects.filter(id__in=document_id_valuelist)
            if clariwebuser.kind == "COR" and not constance.config.CLINIC_SHARE_SERVICES:
                document_qs = document_qs.filter(user=clariwebuser)
            if (clariwebuser.kind == "PAT" and
                constance.config.DEBPT_MINIMUM_AMOUNT > 0):
                # Check for the user debpt
                document_qs = document_qs.filter(
                    Q(is_invoice=True) |
                    Q(user__debpt__lte=constance.config.DEBPT_MINIMUM_AMOUNT))
        if clariwebuser:
            document_qs = document_qs.extra(
                select={'isforeign': "clariweb_document.user_id != %s"},
                select_params=(clariwebuser.pk,))
        return document_qs


def get_document_upload_path(self, filename):
    clariwebuser = self.user
    return os.path.join(
        clariwebuser.user.userdir,
        "demande_%s" % self.sil_demande_number,
        "%s.%s" % (self.document_slug, self.content_type.lower())
    )


def get_structured_result_upload_path(self, filename):
    clariwebuser = self.user
    return os.path.join(
        clariwebuser.user.userdir,
        "demande_{}".format(self.sil_demande_number),
        "{}.json.gz".format(self.document_slug),
    )


class Actor(models.Model):
    last_name = models.CharField(_('last name'), max_length=256, blank=True)
    first_name = models.CharField(_('first name'), max_length=256, blank=True)
    civilite = models.CharField(max_length=128, blank=True)
    date_naissance = models.DateField(blank=True, null=True)

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        if self.civilite:
            full_name = '%s %s' % (self.civilite, full_name)
        return full_name.strip()


class Document(models.Model):
    objects = DocumentManager()

    class Meta:
        verbose_name = "document"
        unique_together = (
            ('sil_demande_pk', 'sil_demande_date', 'user', 'document'),
        )
    CONTENT_TYPES = (
        ('PDF', 'application/pdf'),
        ('JPG', 'image/jpeg'),
    )
    sil_document_pk = models.BigIntegerField(unique=True, null=True, blank=True)
    sil_demande_pk = models.BigIntegerField()
    sil_demande_number = models.CharField(max_length=128)
    sil_demande_date = models.DateTimeField(db_index=True)
    date_add = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(ClariwebUser,
                             related_name="documents_user",
                             db_index=True,
                             on_delete=models.DO_NOTHING)
    datafile = models.FileField(upload_to=get_document_upload_path,
                                storage=documentfs, db_index=True,
                                blank=True, null=True)
    structured_results = models.FileField(
                                upload_to=get_structured_result_upload_path,
                                storage=structuredfs,
                                blank=True, null=True)
    content_type = models.CharField(max_length=128, choices=CONTENT_TYPES)
    document = models.CharField(max_length=128)
    is_invoice = models.BooleanField("Estce une facture", default=False)
    available = models.BooleanField(default=True)
    # Correspondants annexes
    patient = models.ForeignKey(Actor,
                                db_index=True,
                                related_name="documents_patient",
                                on_delete=models.DO_NOTHING)
    medecin = models.ForeignKey(Actor,
                                db_index=True,
                                related_name="documents_medecin",
                                blank=True,
                                null=True,
                                on_delete=models.SET_NULL)
    correspondant = models.ForeignKey(Actor,
                                      db_index=True,
                                      related_name="documents_correspondant",
                                      blank=True,
                                      null=True,
                                      on_delete=models.SET_NULL)
    access_log = models.ManyToManyField(ClariwebUser,
                                        related_name="al+",
                                        through='DocumentAccessLog')
    notify_sms = models.BooleanField(default=False)
    notify_email = models.BooleanField(default=False)
    analyses = JSONField(blank=True, null=True)
    unpaid_amount = models.PositiveIntegerField(default=0, db_index=True)
    invoice_id = models.TextField(db_index=True, blank=True, null=True)
    clinic_ipp = models.CharField(max_length=128, null=True, blank=True)
    clinic_hospi = models.CharField(max_length=128, null=True, blank=True)
    clinic_bed = models.CharField(max_length=128, null=True, blank=True)

    @property
    def document_slug(self):
        return self.document.replace(" ", "_").lower()

    def __str__(self):
        return "%s %s" % (self.sil_demande_number, self.document)

    def read_structured_results(self):
        if not self.structured_results:
            return None
        try:
            return json.loads(self.structured_results.read().decode("utf-8"))
        except:
            logger.exception("structured results json is corrupted")
            return None


class DocumentPerUserManager(models.Manager):
    def foruser(self, user, shareservices=True):
        qs = super().get_queryset()
        clariwebuser = user.clariwebuser
        if not user.is_superuser:
            if not clariwebuser:
                return []
        if not user.is_superuser:
            qs = qs.filter(user=clariwebuser)
        else:
            qs = qs.filter(foreign=False)  # Avoid duplicates
        if not constance.config.CLINIC_SHARE_SERVICES or not shareservices:
            qs = qs.filter(foreign=False)
        qs = qs.select_related("document")
        qs = qs.order_by(
            '-document__sil_demande_date', '-document__sil_demande_number')
        return qs


class DocumentPerUser(models.Model):
    objects = DocumentPerUserManager()

    class Meta:
        verbose_name = "document/user"
        unique_together = ("user", "document")
        indexes = [
            # LC: already handled by unique_together
            # models.Index(fields=['user', 'document']),
            models.Index(fields=['seen_since_modified'])
        ]

    document = models.ForeignKey(Document, on_delete=models.CASCADE)
    user = models.ForeignKey(ClariwebUser, on_delete=models.CASCADE)
    updated = models.DateTimeField(auto_now=True)
    last_seen = models.DateTimeField(null=True, blank=True)
    foreign = models.BooleanField(default=False)
    seen_since_modified = models.BooleanField(default=False)


class DocumentStatisticsPerMonth(models.Model):
    class Meta:
        ordering = ["-timestep", ]
        db_table = "clariweb_stats_documents_per_month"
        managed = False

    timestep = models.DateField(primary_key=True)
    user = models.ForeignKey(ClariwebUser, related_name='+',
                             null=True, blank=True,
                             on_delete=models.DO_NOTHING)
    documents = models.PositiveIntegerField(null=True, blank=True)

    def __str__(self):
        return "%s %s: %s" % (self.timestep, self.user, self.documents)

    def save(self, *args, **kwargs):
        pass

    def delete(self, *args, **kwargs):
        pass


class DocumentAccessLog(models.Model):
    """Document specific access log"""
    class Meta:
        ordering = ["-date", ]

    user = models.ForeignKey(ClariwebUser, db_index=True, related_name="+",
                             on_delete=models.DO_NOTHING)
    document = models.ForeignKey(Document, db_index=True, related_name="+",
                                 on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True, db_index=True)
    ip = models.GenericIPAddressField(unpack_ipv4=True, blank=True, null=True)


class AccessLog(models.Model):
    """Generic access log"""
    class Meta:
        ordering = ["-date", ]

    ACCESSLOG_CHOICES = (
        ('AUTH_OK', "Authentification réussie"),
        ('AUTH_BAD', "Authentification échouée"),
    )
    user = models.ForeignKey(BasicUser, db_index=True, related_name="+",
                             blank=True, null=True,
                             on_delete=models.DO_NOTHING)
    username = models.CharField(max_length=255)
    date = models.DateTimeField(auto_now_add=True, db_index=True)
    message = models.CharField(max_length=32, choices=ACCESSLOG_CHOICES)
    ip = models.GenericIPAddressField(unpack_ipv4=True, blank=True, null=True)

    def save(self, *args, **kwargs):
        if self.user and not self.username:
            self.username = self.user.username
        return super().save(*args, **kwargs)

    def __str__(self):
        return "%s (%s): %s" % (self.user.username, self.ip, self.message)

    def __repr__(self):
        return "<AccessLog: %s: %s>" % (self.user.username, self.message)


class Notification(models.Model):
    STATUS_CHOICES = (
        ("WAITING", "en attente"),
        ("SENT", "envoyé"),
        ("CONFIRMED", "envoi confirmé"),  # SMS only
        ("ERROR", "en erreur"),
    )

    status = models.CharField(max_length=32, choices=STATUS_CHOICES)
    error = models.TextField(blank=True)
    user = models.ForeignKey(ClariwebUser,
                             related_name="+",
                             db_index=True,
                             on_delete=models.DO_NOTHING)
    date = models.DateTimeField(auto_now_add=True, db_index=True)
    sms = models.CharField(max_length=32, blank=True)
    email = models.EmailField(max_length=256, blank=True)
    origin = models.TextField(blank=True)

    def set_origin_from_document(self, document):
        self.origin = "Demande %s du %s" % (
            document.sil_demande_number,
            document.sil_demande_date.strftime("%Y/%m/%d"))

    def save(self, *args, **kwargs):
        assert(self.sms or self.email)
        return super().save(*args, **kwargs)


class PasswordResetManager(models.Manager):
    @staticmethod
    def generate_validation_code(length):
        return ''.join([str(random.choice(list(range(10)))) for _ in range(length)])

    def flush(self):
        self.filter(valid_until__lt=timezone.now()).delete()

    def create(self, user, timeout=240, length=8):
        """Timeout in minutes, length in chars."""
        # Remove existing PasswordReset.
        self.filter(user__pk=user.pk).delete()
        now = timezone.now()
        pr = self.model(
            valid_until=now + datetime.timedelta(seconds=60 * timeout),
            user=user,
            validation_code=self.generate_validation_code(length))
        pr.save()
        return pr


class PasswordReset(models.Model):
    objects = PasswordResetManager()

    valid_until = models.DateTimeField()
    user = models.OneToOneField(BasicUser, unique=True,
                                related_name="+", db_index=True,
                                on_delete=models.DO_NOTHING)
    validation_code = models.CharField(max_length=16)

    @property
    def valid(self):
        return timezone.now() <= self.valid_until


class PaymentRequestQuerySet(models.QuerySet):
    def timedout(self, timeout=600):
        datemax = timezone.now() - datetime.timedelta(seconds=timeout)
        return self.filter(date_insert__lte=datemax, pzresponse=None)


class PaymentRequestManager(models.Manager):
    def get_queryset(self):
        return PaymentRequestQuerySet(self.model, self._db) \
            .select_related('pzresponse')

    def timedout(self, timeout=600):
        return self.get_queryset().timedout(timeout)


class PaymentRequest(payzen_models.PaymentRequest):
    """Replace buggy django_payzen signature algorithm"""

    objects = PaymentRequestManager()
    date_insert = models.DateTimeField(auto_now_add=True)
    pzresponse = models.ForeignKey('PaymentResponse', null=True,
                                   related_name='pzrequest',
                                   on_delete=models.CASCADE)

    @property
    def response(self):
        return self.pzresponse

    def save(self, *args, **kwargs):
        """
        We set up vads_trans_id and theme according to payzen format.

        If fields values are explicitely set by user, we do not override
        their values.
        """
        if not self.vads_trans_date:
            self.vads_trans_date = datetime.datetime.utcnow().replace(
                tzinfo=utc).strftime("%Y%m%d%H%M%S")
        if not self.vads_trans_id:
            self.vads_trans_id = tools.get_vads_trans_id(
                self.vads_site_id, self.vads_trans_date)
        if self.theme and not self.vads_theme_config:
            self.vads_theme_config = str(self.theme)
        if not self.pk:
            super().save()
        self.set_vads_payment_config()
        serializer = kwargs.pop("serializer", None)
        if serializer:
            self.signature = serializer.get_signature(self)
        else:
            self.set_signature()
        super().save(*args, **kwargs)

    def set_signature(self):
        pass

    @property
    def payment_successful(self):
        return self.response.payment_successful if self.response else False


class PaymentResponse(payzen_models.PaymentResponse):
    class Meta:
        proxy = True

    @property
    def request(self):
        try:
            return PaymentRequest.objects.get(
                vads_trans_id=self.vads_trans_id,
                vads_trans_date=self.vads_trans_date,
                vads_site_id=self.vads_site_id)
        except PaymentRequest.DoesNotExist:
            return None


class ClarilabPaymentResponseManager(models.Manager):
    @property
    def errors(self):
        return self.get_queryset().filter(success=False)

    @property
    def success(self):
        return self.get_queryset().filter(success=True)


class ClarilabPaymentResponse(models.Model):
    """PaymentResponse failed to transfer in CLARILAB"""
    objects = ClarilabPaymentResponseManager()

    response = models.OneToOneField(PaymentResponse,
                                    unique=True,
                                    on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now=True)
    reason = models.TextField()
    success = models.BooleanField()


# Don't load inside __init__.py. Otherwise gunicorn will crash trying to
# import clariweb module.
from .signals import (
    auto_delete_file_on_delete,
    notify_user,
    email_confirmation,
    auto_delete_userdir_on_delete,
    update_user_email,
    create_passwordreset_send_validationcode,
    create_clarilab_reglement_task
)  # noqa
