# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals

import os
import shutil
import logging
import datetime
import constance
from django.db import models
from django.conf import settings
from django.dispatch import receiver
from django.template.loader import get_template
from django.utils import timezone
from .models import (
    BasicUser,
    ClariwebUser,
    Document,
    Notification,
    PasswordReset,
    PaymentResponse,
)
from clariweb.backend import tasks


logger = logging.getLogger("clariweb.models")


@receiver(models.signals.post_delete, sender=Document)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    if instance.datafile:
        # LC: save=False is required:
        # document.delete() will automatically remove dafatile
        #   -> call the post_save signal on Document (created=True!!)
        #   -> send a notification email.
        instance.datafile.delete(save=False)


def check_sms_send(user):
    """
    Check constance settings to know if we should send a notification or not.
    """
    if user.kind == "PAT":
        return constance.config.SMS_NOTIFICATION_PATIENT
    elif user.kind == "MED":
        return constance.config.SMS_NOTIFICATION_MEDECIN
    elif user.kind == "COR":
        return constance.config.SMS_NOTIFICATION_CORRESPONDANT
    elif user.kind == "PREL":
        return constance.config.SMS_NOTIFICATION_PRELEVEUR
    return False


@receiver(models.signals.post_save, sender=Document)
def notify_user(sender, instance, **kwargs):
    document = instance
    # Si on créée le document pour un transmetteur, sans disposer du PDF,
    # on ne souhaite pas envoyer de notification
    if not document.available:
        return False
    # Pas de notification pour les modifications trop anciennes
    if not kwargs["created"]:
        delta = timezone.now() - instance.date_add
        max_delta = datetime.timedelta(
            constance.config.NOTIFICATION_RESEND_MAXIMUM_DELAY)
        if delta >= max_delta:
            return False
    user = document.user
    if not user.user.is_active:
        logger.debug("Won't send notification to a disabled user (%s).",
                     user.user)
        return False
    # On ne veut pas envoyer deux notifications au même utilisateur
    # dans un intervale trop court.
    if constance.config.NOTIFICATION_SAMEUSER_MINIMUM_DELAY > 0:
        delay_hours = constance.config.NOTIFICATION_SAMEUSER_MINIMUM_DELAY
        no_multiple_notification_minimum_date = (
            timezone.now() - datetime.timedelta(
                seconds=delay_hours * 3600)
        )
        notifications = Notification.objects.filter(
            user=user,
            date__gte=no_multiple_notification_minimum_date)
        if document.notify_sms and user.address_mobile_phone:
            notifications.filter(sms="{}".format(user.address_mobile_phone))
        if document.notify_email and user.user.email:
            notifications.filter(email=user.user.email)
        if notifications.count() > 0:
            return False
    if (document.notify_sms and user.address_mobile_phone and
            check_sms_send(user)):
        if user.kind == "PAT":
            template_message = get_template("sms/notification_patient.txt")
        elif user.kind == "PREL":
            template_message = get_template("sms/notification_preleveur.txt")
        else:
            template_message = get_template("sms/notification_medecin.txt")
        context = {
            "document": document,
            "patient": document.patient,
            "user": user,
            "customization": constance.config,
        }
        notification = Notification(
            user=user, status="WAITING",
            sms="{}".format(user.address_mobile_phone))
        notification.set_origin_from_document(document)
        notification.save()
        tasks.send_sms_notification(
            notification_id=notification.pk,
            numbers=[notification.sms],
            message=template_message.render(context))
    if document.notify_email and user.user.email:
        send_to_unconfirmed = constance.config.EMAIL_NOTIFICATION_UNCONFIRMED
        if ((not send_to_unconfirmed and user.user.is_confirmed) or
                send_to_unconfirmed):
            email = user.user.email
            if user.kind == "PAT":
                template_body = get_template(
                    "email/notification_patient.txt")
                template_subject = get_template(
                    "email/notification_patient_subject.txt")
            else:
                template_body = get_template(
                    "email/notification_medecin.txt")
                template_subject = get_template(
                    "email/notification_medecin_subject.txt")
            context = {
                "document": document,
                "patient": document.patient,
                "user": user,
                "customization": constance.config,
            }
            notification = Notification(
                user=user, status="WAITING", email=email)
            notification.set_origin_from_document(document)
            notification.save()
            tasks.send_email_notification(
                notification_id=notification.pk,
                emails=[email],
                subject=template_subject.render(context),
                body=template_body.render(context))


@receiver(models.signals.post_save, sender=Document)
def email_confirmation(sender, instance, **kwargs):
    if instance.document == 'Compte-Rendu PDF' and not instance.is_invoice:
        buser = instance.user.user
        queryset = buser.email_address_set.filter(
            email=buser.email, email_sent=False, confirmed_at=None).exists()
        if queryset:
            buser.send_email_confirmation()


@receiver(models.signals.post_delete, sender=ClariwebUser)
def auto_delete_userdir_on_delete(sender, instance, **kwargs):
    userdir = os.path.join(settings.DOCUMENTS_ROOT,
                           "user_%d" % instance.pk)
    if os.path.isdir(userdir):
        shutil.rmtree(userdir)


@receiver(models.signals.post_save, sender=BasicUser)
def update_user_email(sender, instance, **kwargs):
    user = instance
    user.email_address_set.exclude(email=user.email).delete()
    if user.email:
        try:
            user.add_unconfirmed_email(user.email)
        except:
            logger.error(
                "User email %s is invalid. Not sending email.",
                user.email)


@receiver(models.signals.post_save, sender=PasswordReset)
def create_passwordreset_send_validationcode(sender, instance, **kwargs):
    pr = instance
    clariwebuser = pr.user.clariwebuser
    mobilenumber = "{}".format(clariwebuser.address_mobile_phone)
    notification = Notification(
        user=clariwebuser, status="WAITING", sms=mobilenumber)
    notification.origin = 'passwordreset'
    notification.save()
    tasks.send_sms_notification(
        notification_id=notification.pk,
        numbers=[notification.sms],
        message="Votre code de validation CLARIWEB est %s." % (
            pr.validation_code, )
    )


@receiver(models.signals.post_save, sender=PaymentResponse,
          dispatch_uid="paymentresponse_clarilab_reglement")
def create_clarilab_reglement_task(sender, instance, **kwargs):
    resp = instance
    if kwargs["created"] and resp.payment_successful:
        tasks.clarilab_reglement.delay(resp.pk)
