# -*- coding: utf-8 -*-

import datetime
from django.utils import timezone
from django.test import TestCase
from .models import (
    BasicUser,
    ClariwebUser,
    Notification,
    PasswordReset
)


class PasswordResetTestCase(TestCase):
    def setUp(self):
        self.u1 = BasicUser.objects.create_user(
            username="user1", password="user1")
        clariwebuser = ClariwebUser(
            sil_pk=1, address_mobile_phone="+336000000000")
        clariwebuser.save()
        self.u1.clariwebuser = clariwebuser
        self.u1.save()
        self.u2 = BasicUser.objects.create_user(
            username="user2", password="user2")
        clariwebuser = ClariwebUser(
            sil_pk=2, address_mobile_phone="+336000000001")
        clariwebuser.save()
        self.u2.clariwebuser = clariwebuser
        self.u2.save()

    def test_single_reset(self):
        PasswordReset.objects.create(self.u1)
        pr = PasswordReset.objects.get(user__pk=self.u1.pk)
        self.assertEqual(len(pr.validation_code), 8)
        notifications = Notification.objects.all()
        self.assertEqual(len(notifications), 1)
        self.assertEqual(notifications[0].origin, 'passwordreset')

    def test_single_reset_length10(self):
        pr = PasswordReset.objects.create(self.u1, length=10)
        self.assertEqual(len(pr.validation_code), 10)

    def test_multi_reset(self):
        PasswordReset.objects.create(self.u1, length=10)
        PasswordReset.objects.create(self.u2)
        pr = PasswordReset.objects.create(self.u1)
        self.assertEqual(len(pr.validation_code), 8)
        self.assertEqual(
            len(PasswordReset.objects.filter(user__pk=self.u1.pk)), 1)
        self.assertEqual(len(PasswordReset.objects.all()), 2)

    def test_timeout_default(self):
        now = timezone.now()
        pr = PasswordReset.objects.create(self.u1)
        timeout = now + datetime.timedelta(seconds=60 * 14)
        self.assertTrue(pr.valid_until >= timeout,
                        (pr.valid_until, timeout))

    def test_timeout_custom(self):
        pr = PasswordReset.objects.create(self.u1, timeout=20)
        timeout = timezone.now() + datetime.timedelta(seconds=60 * 19)
        self.assertTrue(pr.valid_until >= timeout,
                        (pr.valid_until, timeout))

    def test_reset_timeout(self):
        pr = PasswordReset.objects.create(self.u1)
        pr.valid_until = timezone.now() - datetime.timedelta(1)
        self.assertFalse(pr.valid)

    def flush_timeout(self):
        pr = PasswordReset.objects.create(self.u1)
        pr.save()
        pr = PasswordReset.objects.create(self.u2)
        pr.valid_until = timezone.now() - datetime.timedelta(1)
        pr.save()
        self.assertEqual(len(PasswordReset.objects.all()), 2)
        pr.objects.flush()
        self.assertEqual(len(PasswordReset.objects.all()), 1)
