#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import logging.config
import argparse
import multiprocessing
from pathlib import Path
from .http import ClariwebHTTPServer
from .newconfig import write_default_settings


def run_django(command, args):
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "clariweb.settings")
    from django.core.management import execute_from_command_line
    argv = [sys.argv[0] + ' django', command] + list(args)
    execute_from_command_line(argv=argv)


def run_webserver(args):
    socket = Path(args.unix).absolute() if args.unix else None
    bindaddress = socket if socket else args.tcp
    extra_options = {
        'http-socket': bindaddress,
        'evil-reload-on-rss': args.max_rss,
        "py-autoreload": 1 if args.autoreload else 0,
    }
    server = ClariwebHTTPServer(
        workers=args.workers,
        extra_options=extra_options
    )
    server.run()


def run_worker(args):
    from clariweb.celery import app
    from django.conf import settings
    settings.DEBUG = False
    worker = app.Worker(
        pool_cls='processes',
        logfile=os.path.join(settings.LOG_DIR, "celery.log"),
        max_memory_per_child=args.max_rss * 1024,
        concurrency=args.workers)
    worker.start()
    sys.exit(worker.exitcode)


def run_beat(args):
    from clariweb.celery import app
    from django.conf import settings
    settings.DEBUG = False
    app.Beat(
        logfile=os.path.join(settings.LOG_DIR, "celery.log")
    ).run()


def run_create_config(args):
    config_path = Path("~/.clariweb/clariweb.conf.py").expanduser()
    doc_path = Path("~/.clariweb/documents").expanduser()
    if not doc_path.exists():
        doc_path.mkdir(parents=True)
    if not config_path.exists():
        write_default_settings(config_path)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "clariweb.settings")
    from django.conf import settings

def run_readme(args):
    from pkg_resources import resource_string
    for l in resource_string(__name__, '../README.md').decode("utf-8").splitlines():
        print(l)


def main():
    from .prctl import set_pdeathsig
    import signal
    set_pdeathsig(signal.SIGTERM)

    formatter_class = argparse.ArgumentDefaultsHelpFormatter
    parser = argparse.ArgumentParser(
        "clariweb", formatter_class=formatter_class)
    subparsers = parser.add_subparsers(dest="subcmd")
    webparser = subparsers.add_parser(
        "web", help="web runner (uwsgi)", formatter_class=formatter_class)
    listengroup = webparser.add_mutually_exclusive_group()
    listengroup.add_argument(
        "--unix", default=None, help="Unix socket listener")
    listengroup.add_argument(
        "--tcp", default="[::1]:8002", help="TCP socket bind address")
    webparser.add_argument(
        "--workers", type=int, default=min(multiprocessing.cpu_count(), 4),
        help="Max worker count")
    webparser.add_argument(
        "--max-rss", type=int, default=1024,
        help="Kill worker if RSS is greater than specified RSS in MB")
    webparser.add_argument(
        "--autoreload", action="store_true",
        help="reload if python module changes")
    webparser.set_defaults(func=run_webserver)

    workerparser = subparsers.add_parser(
        "worker", help="celery worker (async tasks)",
        formatter_class=formatter_class)
    workerparser.add_argument(
        "--max-rss", type=int, default=1024,
        help="Kill worker if RSS is greather tha specified RSS in MB")
    workerparser.add_argument(
        "--workers", type=int, default=min(multiprocessing.cpu_count(), 2),
        help="Max worker count")
    workerparser.set_defaults(func=run_worker)

    beatparser = subparsers.add_parser(
        "beat", help="celery beat (periodic tasks)",
        formatter_class=formatter_class)
    beatparser.set_defaults(func=run_beat)

    ccparser = subparsers.add_parser(
        "createconfig", help="create new configuration",
        formatter_class=formatter_class)
    ccparser.set_defaults(func=run_create_config)

    djangoparser = subparsers.add_parser(
        "django", help="run django command")
    djangoparser.add_argument('command')
    djangoparser.add_argument('args', nargs=argparse.REMAINDER)

    readmep = subparsers.add_parser("readme", help="Print README.md")
    readmep.set_defaults(func=run_readme)

    args = parser.parse_args()
    if hasattr(args, "func"):
        args.func(args)
    else:
        if 'command' not in args:
            parser.print_help()
            sys.exit(1)
        else:
            run_django(args.command, args.args)


if __name__ == "__main__":
    main()
