import os
import sys
import shutil
import datetime
import traceback

from setuptools import Command
from subprocess import check_output
from distutils import log


class BuildAssetsCommand(Command):
    boolean_options = ['force']
    user_options = [
        ('work-path=', 'w', "The working directory for source files. Defaults to ."),
        ('build-lib=', 'b', "directory for script runtime modules"),
        (
            'force', 'f', "Force rebuilding of static content. Defaults to rebuilding on version "
            "change detection."
        ),
        (
            'force', 'f', "Force rebuilding of static content. Defaults to rebuilding on version "
            "change detection."
        ),
    ]

    description = 'build static media assets'

    def initialize_options(self):
        self.build_lib = None
        self.force = None
        self.work_path = None

    def get_dist_paths(self):
        return ['clariweb/webui/dist']

    def _get_package_version(self):
        pkg_path = os.path.join(self.work_path)
        sys.path.insert(0, pkg_path)
        try:
            import clariweb
        except:
            version = None
            build = None
            import traceback
            traceback.print_exc()
        else:
            log.info(u'pulled version information from \'clariweb\' module'.format(clariweb.__file__))
            version = self.distribution.get_version()
            build = clariweb.__build__
        finally:
            sys.path.pop(0)
        return {
            'version': version,
            'build': build,
        }

    def _needs_static(self, version_info):
        return True

    def _needs_built(self):
        for path in self.get_dist_paths():
            if not os.path.isdir(path):
                return True
        version_info = self._get_package_version()
        return self._needs_static(version_info)

    def _setup_npm(self):
        node_version = []
        for app in 'node', 'npm':
            try:
                node_version.append(self._run_command([app, '--version']).rstrip())
            except OSError:
                log.fatal(
                    u'Cannot find `{0}` executable. Please install {0}`'
                    ' and try again.'.format(app)
                )
                sys.exit(1)
        log.info(u'using node ({0}) and npm ({1})'.format(*node_version))
        self._run_command(['npm', 'install', '--production', '--quiet'])

    def _run_command(self, cmd, env=None):
        log.debug('running [%s]' % (' '.join(cmd), ))
        try:
            return check_output(cmd, cwd=os.path.join(self.work_path, "clariweb", "webui"), env=env)
        except:
            log.error('command failed [%s] via [%s]' % (' '.join(cmd), self.work_path, ))
            raise

    def _build(self):
        version_info = self._get_package_version()
        log.info(
            u'building assets for {} v{} (build {})'.format(
                self.distribution.get_name(),
                version_info['version'] or 'UNKNOWN',
                version_info['build'] or 'UNKNOWN',
            )
        )
        if not version_info['version'] or not version_info['build']:
            log.fatal('Could not determine clariweb version or build')
            sys.exit(1)

        try:
            self._build_static()
        except:
            traceback.print_exc()
            log.fatal(
                'unable to build Clariweb\'s static assets!\n'
                'Hint: You might be running an invalid version of NPM.'
            )
            sys.exit(1)

    def _build_static(self):
        env = dict(os.environ)
        env["CLARIWEB_STATIC_DIST_PATH"] = os.path.abspath(
            os.path.join(self.build_lib, 'clariweb', 'webui', 'dist'))
        env['NODE_ENV'] = 'production'
        self._run_command(['npm', 'run', 'prod'], env=env)

    def get_root_path(self):
        return os.path.abspath(os.path.dirname(sys.modules['__main__'].__file__))

    def finalize_options(self):
        sdist = self.distribution.get_command_obj('sdist')
        build_ext = self.get_finalized_command('build_ext')
        self.inplace = (build_ext.inplace or sdist.finalized) and 1 or 0
        if self.inplace:
            log.info('in-place js building enabled')
            self.build_lib = '.'
        # If we're coming from sdist, clear the hell out of the dist
        # folder first.
        if sdist.finalized:
            for path in self.get_dist_paths():
                try:
                    shutil.rmtree(path)
                except (OSError, IOError):
                    pass
        self.set_undefined_options('build', ('build_lib', 'build_lib'))
        log.debug('regular js build: build path is %s' % self.build_lib)
        if self.work_path is None:
            self.work_path = self.get_root_path()

    def update_manifests(self):
        # if we were invoked from sdist, we need to inform sdist about
        # which files we just generated.  Otherwise they will be missing
        # in the manifest.  This adds the files for what webpack generates
        # plus our own assets.json file.
        sdist = self.distribution.get_command_obj('sdist')
        if not sdist.finalized:
            return

        # The path down from here only works for sdist:

        # Use the underlying file list so that we skip the file-exists
        # check which we do not want here.
        files = sdist.filelist.files
        base = os.path.abspath('.')

        # We need to split off the local parts of the files relative to
        # the current folder.  This will chop off the right path for the
        # manifest.
        for path in self.get_dist_paths():
            for dirname, _, filenames in os.walk(os.path.abspath(path)):
                for filename in filenames:
                    filename = os.path.join(dirname, filename)
                    files.append(filename[len(base):].lstrip(os.path.sep))

    def run(self):
        if os.environ.get("LIGHT_BUILD"):
            return
        else:
            if self.force or self._needs_built():
                self._setup_npm()
                self._build()
                self.update_manifests()
