'use strict';

// Modules
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const ManifestPlugin = require('webpack-manifest-plugin');
var version = require("./package.json").version;
var path = require('path');

if (process.env.CLARIWEB_STATIC_DIST_PATH) {
    var distPath = process.env.CLARIWEB_STATIC_DIST_PATH;
} else {
    var distPath = path.resolve(__dirname, 'dist');
}

module.exports = (env, argv) => {
  /**
   * Config
   * Reference: http://webpack.github.io/docs/configuration.html
   * This is the object where all configuration gets set
   */
  var isProd = argv.mode === 'production';
  var config = {};

  /**
   * Entry
   * Reference: http://webpack.github.io/docs/configuration.html#entry
   * Should be an empty object if it's generating a test build
   * Karma will set this when it's a test build"
   */
  config.entry = {
    'app': './src/app/js/app.js',
    'pdf.worker': 'pdfjs-dist/build/pdf.worker.entry',
    'theme-cosmo': './src/less/themes/cosmo.less',
    'theme-cyborg': './src/less/themes/cyborg.less',
    'theme-darkly': './src/less/themes/darkly.less',
    'theme-flatly': './src/less/themes/flatly.less',
    'theme-journal': './src/less/themes/journal.less',
    'theme-lumen': './src/less/themes/lumen.less',
    'theme-main': './src/less/main.less',
    'theme-paper': './src/less/themes/paper.less',
    'theme-readable': './src/less/themes/readable.less',
    'theme-sandstone': './src/less/themes/sandstone.less',
    'theme-simplex': './src/less/themes/simplex.less',
    'theme-slate': './src/less/themes/slate.less',
    'theme-spacelab': './src/less/themes/spacelab.less',
    'theme-united': './src/less/themes/united.less',
    'theme-yeti': './src/less/themes/yeti.less'
  };

  /**
   * Output
   * Reference: http://webpack.github.io/docs/configuration.html#output
   * Should be an empty object if it's generating a test build
   * Karma will handle setting it up for you when it's a test build
   */
  config.output = {
    path: distPath,
    publicPath: '',
    filename: isProd ? '[name].[hash].js' : '[name].bundle.js',
    chunkFilename: isProd ? '[name].[hash].js' : '[name].bundle.js'
  };

  config.devtool = 'source-map';

  // Initialize module
  config.module = {
    rules: [
      {
        test: /\.js$/,
        use: [
          { loader: 'ng-annotate-loader?ngAnnotate=ng-annotate-patched' },
          {
            loader: 'babel-loader',
            options: {
              presets: [
                ['env', {
                  targets: { browsers: ["chrome>=38"] },
                }]
              ]
            }
          },
        ],
        exclude: /node_modules/
      }, {
        test: /\.less$/,
        use: [
          MiniCssExtractPlugin.loader,
          'css-loader',
          'less-loader',
        ]
      }, {
        test: /\.(ico|png|jpg|jpeg|gif)$/,
        loader: 'file-loader',
        options: {
          outputPath: 'images/',
          name: isProd ? '[name].[hash].[ext]' : '[name].[ext]'
        }
      }, {
        test: /\.(woff|woff2|ttf|eot|svg)(\?.+$|$)$/,
        use: [{
          loader: 'file-loader',
          options: {
            name: isProd ? '[name].[hash].[ext]' : '[name].[ext]',
            outputPath: 'fonts/',
            rel: distPath
          }
        }]
      }, {
        test: /\.tpl\.html$/,
        use: [{
          loader: 'html-loader',
          options: {
            interpolate: true
          }
        }]
      }
    ]
  };

  config.optimization = {
    minimize: false,
    minimizer: isProd ? [new UglifyJsPlugin()] : [],
    splitChunks: {
      cacheGroups: {
        vendors: {
          name: 'vendors',
          chunks: 'all',
          reuseExistingChunk: true,
          priority: 1,
          enforce: true,
          test(module, chunks) {
              const name = module.nameForCondition && module.nameForCondition();
              return chunks.some(chunk => {
                  return chunk.name === 'app' && /[\\/]node_modules[\\/]/.test(name);
              });
          }
        }
      }
    }
  };

  config.plugins = [
    new CopyWebpackPlugin([{
      from: './src/assets/oldie.html',
      to: 'assets/'
    }]),
    new HtmlWebpackPlugin({
      template: './src/index.html',
      chunks: ['app', 'pdf.worker', 'vendors'],
      inject: 'body',
      version: version
    }),
    new MiniCssExtractPlugin({
      filename: isProd ? '[name].[hash].css' : '[name].css',
      chunkFilename: isProd ? '[id].[hash].css' : '[id].css'
    }),
    new ManifestPlugin({
      fileName: 'themes-manifest.json',
      filter: (i) => { return i.name.startsWith("theme-") && i.name.endsWith(".css"); }
    })
  ];
  return config;
};